@@ -26,7 +26,7 @@
 from __future__ import annotations
 
 import traceback
-from typing import TYPE_CHECKING, List
+from typing import TYPE_CHECKING, List, Dict
 
 from fabric_cf.actor.boot.configuration_exception import ConfigurationException
 from fabric_cf.actor.boot.inventory.aggregate_resource_model_creator import AggregateResourceModelCreator
@@ -317,7 +317,7 @@ class ConfigurationProcessor:
             self.resources = self.read_resource_config(config=config)
 
     @staticmethod
-    def read_resource_config(*, config: ActorConfig) -> dict:
+    def read_resource_config(*, config: ActorConfig) -> Dict[ResourceType, ResourceConfig]:
         """
         Read resource config and create ARM and inventory slices
         @param config actor config
@@ -329,17 +329,18 @@ class ConfigurationProcessor:
             return result
 
         for r in resources:
-            descriptor = ResourceConfig()
-            descriptor.set_resource_type(rtype=ResourceType(resource_type=r.get_type()))
-            descriptor.set_resource_type_label(rtype_label=r.get_label())
+            for resource_type in r.get_type():
+                descriptor = ResourceConfig()
+                descriptor.set_resource_type(rtype=ResourceType(resource_type=resource_type))
+                descriptor.set_resource_type_label(rtype_label=r.get_label())
 
-            handler = r.get_handler()
-            if handler is not None:
-                descriptor.set_handler_class(handler_class=handler.get_class_name())
-                descriptor.set_handler_module(module=handler.get_module_name())
-                descriptor.set_handler_properties(properties=handler.get_properties())
+                handler = r.get_handler()
+                if handler is not None:
+                    descriptor.set_handler_class(handler_class=handler.get_class_name())
+                    descriptor.set_handler_module(module=handler.get_module_name())
+                    descriptor.set_handler_properties(properties=handler.get_properties())
 
-            result[descriptor.get_resource_type()] = descriptor
+                result[descriptor.get_resource_type()] = descriptor
         return result
 
     def initialize_actor(self):
