@@ -36,22 +36,12 @@ if TYPE_CHECKING:
     from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
     from fabric_cf.actor.core.kernel.resource_set import ResourceSet
     from fabric_cf.actor.security.auth_token import AuthToken
-    from fabric_cf.actor.core.apis.abc_delegation import ABCDelegation
 
 
 class ABCAuthority(ABCServerActor):
     """
     IAuthority defines the interface for an actor acting in the authority role.
     """
-    @abstractmethod
-    def donate_delegation(self, *, delegation: ABCDelegation):
-        """
-        Accepts delegations to be merged to Model.
-
-        @param delegation delegation
-
-        @raises Exception in case of error
-        """
 
     @abstractmethod
     def eject(self, *, resources: ResourceSet):
