@@ -165,7 +165,7 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_reservation(self, *, rid: ID, oidc_claim_sub: str = None) -> ABCReservationMixin:
+    def get_reservation(self, *, rid: ID, oidc_claim_sub: str = None) -> ABCReservationMixin or None:
         """
         Retrieves the specified reservation record.
 
@@ -201,6 +201,20 @@ class ABCDatabase(ABC):
         @throws Exception in case of error
         """
 
+    @abstractmethod
+    def get_reservations_by_graph_node_id_state(self, *, graph_node_id: str,
+                                                states: List[int]) -> List[ABCReservationMixin]:
+        """
+        Retrieves the specified reservations which correspond to a specific graph node
+
+        @param graph_node_id graph_node_id
+        @param states list of states
+
+        @return list of properties for reservations
+
+        @throws Exception in case of error
+        """
+
     @abstractmethod
     def get_reservations_by_slice_id_state(self, *, slice_id: ID, state: int) -> List[ABCReservationMixin]:
         """
@@ -236,7 +250,7 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_slice(self, *, slice_id: ID) -> ABCSlice:
+    def get_slice(self, *, slice_id: ID) -> ABCSlice or None:
         """
         Retrieves the specified slice record.
 
@@ -248,12 +262,13 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_slice_by_name(self, *, slice_name: str, oidc_claim_sub: str) -> List[ABCSlice]:
+    def get_slice_by_name(self, *, slice_name: str, oidc_claim_sub: str, email: str) -> List[ABCSlice] or None:
         """
         Retrieves the specified slice record.
 
         @param slice_name slice name
         @param oidc_claim_sub User OIDC Sub Claim
+        @param email User email
 
         @return dict of properties
 
@@ -261,7 +276,32 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_slice_by_resource_type(self, *, rtype: ResourceType) -> ABCSlice:
+    def get_slice_by_oidc_claim_sub(self, *, oidc_claim_sub: str) -> List[ABCSlice] or None:
+        """
+        Retrieves the specified slice record.
+
+        @param oidc_claim_sub User OIDC Sub Claim
+        @param email User email
+
+        @return dict of properties
+
+        @throws Exception in case of error
+        """
+
+    @abstractmethod
+    def get_slice_by_email(self, *, email: str) -> List[ABCSlice] or None:
+        """
+        Retrieves the specified slice record.
+
+        @param email User email
+
+        @return dict of properties
+
+        @throws Exception in case of error
+        """
+
+    @abstractmethod
+    def get_slice_by_resource_type(self, *, rtype: ResourceType) -> ABCSlice or None:
         """
         Retrieves the specified slice record.
 
@@ -273,7 +313,7 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_slices(self) -> List[ABCSlice]:
+    def get_slices(self) -> List[ABCSlice] or None:
         """
         Retrieves all slice records.
 
@@ -284,7 +324,7 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_inventory_slices(self) -> List[ABCSlice]:
+    def get_inventory_slices(self) -> List[ABCSlice] or None:
         """
         Retrieves all inventory slice records.
 
@@ -295,7 +335,7 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_client_slices(self) -> List[ABCSlice]:
+    def get_client_slices(self) -> List[ABCSlice] or None:
         """
         Retrieves all client slice records.
 
@@ -364,18 +404,18 @@ class ABCDatabase(ABC):
 
         @param state state
 
-        @return list of properties for reservations
+        @return list of reservations
 
         @throws Exception in case of error
         """
 
     @abstractmethod
-    def get_reservations_by_rids(self, *, rid: List[str]):
+    def get_reservations_by_rids(self, *, rid: List[str]) -> List[ABCReservationMixin]:
         """
         Retrieves the specified reservation records.
         The order in the return vector is the same order as @rids
         @param rid rids
-        @return list of properties
+        @return list of reservations
         @throws Exception in case of error
         """
 
@@ -401,7 +441,7 @@ class ABCDatabase(ABC):
         """
 
     @abstractmethod
-    def get_delegation(self, *, dlg_graph_id: str) -> ABCDelegation:
+    def get_delegation(self, *, dlg_graph_id: str) -> ABCDelegation or None:
         """
         Get Delegation
         @params dlg_graph_id: dlg_graph_id
@@ -438,3 +478,9 @@ class ABCDatabase(ABC):
         """
         Return all config mappings
         """
+
+    @abstractmethod
+    def get_brokers(self) -> List[ABCBrokerProxy] or None:
+        """
+        Return all brokers
+        """
