@@ -233,6 +233,18 @@ class ABCDelegation(ABC):
         Returns true if delegation is closed; false otherwise
         """
 
+    @abstractmethod
+    def is_delegated(self) -> bool:
+        """
+        Returns true if delegation is delegated; false otherwise
+        """
+
+    @abstractmethod
+    def is_reclaimed(self) -> bool:
+        """
+        Returns true if delegation is reclaimed; false otherwise
+        """
+
     @abstractmethod
     def is_failed(self) -> bool:
         """
