@@ -102,6 +102,7 @@ class Constants:
     PROPERTY_CONF_KAFKA_SASL_CONSUMER_USERNAME = "kafka-sasl-consumer-username"
     PROPERTY_CONF_KAFKA_SASL_CONSUMER_PASSWORD = "kafka-sasl-consumer-password"
     PROPERTY_CONF_KAFKA_SASL_MECHANISM = "kafka-sasl-mechanism"
+    PROPERTY_CONF_KAFKA_REQUEST_TIMEOUT_MS = "request.timeout.ms"
 
     KAFKA_TOPIC = "kafka-topic"
     NAME = "name"
@@ -112,6 +113,7 @@ class Constants:
     HANDLER = "handler"
     CREDMGR_HOST = "credmgr-host"
     PUBLISH_INTERVAL = "publish-interval"
+    REFRESH_INTERVAL = "refresh-interval"
     DELEGATION = "delegation"
 
     PROPERTY_CLASS_NAME = "ObjectClassName"
@@ -127,7 +129,8 @@ class Constants:
 
     PROPERTY_CONF_PROMETHEUS_REST_PORT = "prometheus.port"
     PROPERTY_CONF_CONTROLLER_REST_PORT = "orchestrator.rest.port"
-    PROPERTY_CONF_CONTROLLER_CREATE_WAIT_TIME_MS = "orchestrator.create.wait.time.ms"
+    PROPERTY_CONF_CONTROLLER_CREATE_WAIT_TIME = "orchestrator.create.wait.time"
+    PROPERTY_CONF_RPC_REQUEST_TIMEOUT_SECONDS = "rpc.request.timeout.seconds"
 
     PROPERTY_SUBSTRATE_FILE = "substrate.file"
     PROPERTY_AGGREGATE_RESOURCE_MODEL = "AggregateResourceModel"
@@ -138,6 +141,7 @@ class Constants:
 
     CONFIG_LOGGING_SECTION = 'logging'
     PROPERTY_CONF_LOG_FILE = 'log-file'
+    PROPERTY_CONF_HANDLER_LOG_FILE = 'handler-log-file'
     PROPERTY_CONF_LOG_LEVEL = 'log-level'
     PROPERTY_CONF_LOG_RETAIN = 'log-retain'
     PROPERTY_CONF_LOG_SIZE = 'log-size'
@@ -164,6 +168,7 @@ class Constants:
     PROPERTY_CONF_DB_HOST = "db-host"
 
     CONFIG_SECTION_NEO4J = "neo4j"
+    CONFIG_SECTION_BQM = "bqm"
 
     PROPERTY_CONF_MODULE_NAME = 'module'
     PROPERTY_CONF_CLASS_NAME = 'class'
@@ -174,6 +179,8 @@ class Constants:
 
     HOME_DIRECTORY = '/usr/src/app/'
     SUPERBLOCK_LOCATION = HOME_DIRECTORY + "state_recovery.lock"
+    MAINTENANCE_LOCATION = HOME_DIRECTORY + "maintenance.lock"
+    MODEL_RELOAD_LOCATION = HOME_DIRECTORY + "reload.model"
     CONFIGURATION_FILE = "/etc/fabric/actor/config/config.yaml"
     STATE_FILE_LOCATION = '/tmp/fabric_actor.tmp'
 
@@ -236,7 +243,11 @@ class Constants:
 
     UNSUPPORTED_RESOURCE_TYPE = "Unsupported resource type: {}"
 
+    CLOSURE_BY_TICKET_REVIEW_POLICY = "TicketReviewPolicy: Closing reservation due to failure in slice"
+    MAINTENANCE_MODE_ERROR = "Testbed is in maintenance mode, Create/Renew Slice(s) are disabled!"
+
     CLAIMS_SUB = "sub"
+    CLAIMS_EMAIL = "email"
 
     PROPERTY_EXCEPTION_MESSAGE = "exception.message"
     PROPERTY_TARGET_NAME = "target.name"
@@ -259,5 +270,15 @@ class Constants:
     # Orchestrator Lease params
     TWO_WEEKS = timedelta(days=30)
     DEFAULT_MAX_DURATION = TWO_WEEKS
-    RENEW_TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
+    LEASE_TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
     DEFAULT_LEASE_IN_HOURS = 24
+
+    STATE_ALL = "ALL"
+
+    # Default offset used to pick a VLAN from the range 1-4096
+    DEFAULT_VLAN_OFFSET = 10
+    VLAN_START = 1
+    VLAN_END = 4096
+
+    CONFIG_PROPERTIES_FILE = "config.properties.file"
+    MODE = "mode"
