@@ -28,7 +28,7 @@ import threading
 import traceback
 from typing import List
 
-from fabric_cf.actor.core.apis.abc_delegation import ABCDelegation
+from fabric_cf.actor.core.apis.abc_delegation import ABCDelegation, DelegationState
 from fabric_cf.actor.core.apis.abc_policy import ABCPolicy
 from fabric_cf.actor.core.apis.abc_timer_task import ABCTimerTask
 from fabric_cf.actor.core.apis.abc_actor_mixin import ABCActorMixin, ActorType
@@ -40,6 +40,7 @@ from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
 from fabric_cf.actor.core.apis.abc_slice import ABCSlice
 from fabric_cf.actor.core.common.exceptions import ActorException
 from fabric_cf.actor.core.container.message_service import MessageService
+from fabric_cf.actor.core.delegation.delegation_factory import DelegationFactory
 from fabric_cf.actor.core.kernel.failed_rpc import FailedRPC
 from fabric_cf.actor.core.kernel.kernel_wrapper import KernelWrapper
 from fabric_cf.actor.core.kernel.rpc_manager_singleton import RPCManagerSingleton
@@ -242,7 +243,7 @@ class ActorMixin(ABCActorMixin):
         self.wrapper.extend_reservation(rid=rid, resources=resources, term=term)
 
     def external_tick(self, *, cycle: int):
-        self.logger.debug("External Tick start cycle: {}".format(cycle))
+        self.logger.info("External Tick start cycle: {}".format(cycle))
 
         class TickEvent(ABCActorEvent):
             def __init__(self, *, base, cycle: int):
@@ -256,7 +257,7 @@ class ActorMixin(ABCActorMixin):
                 self.base.actor_tick(cycle=self.cycle)
 
         self.queue_event(incoming=TickEvent(base=self, cycle=cycle))
-        self.logger.debug("External Tick end cycle: {}".format(cycle))
+        self.logger.info("External Tick end cycle: {}".format(cycle))
 
     def actor_tick(self, *, cycle: int):
         """
@@ -275,7 +276,7 @@ class ActorMixin(ABCActorMixin):
                 current_cycle = self.current_cycle + 1
 
             while current_cycle <= cycle:
-                self.logger.debug("actor_tick: {} start".format(current_cycle))
+                self.logger.info("actor_tick: {} start".format(current_cycle))
                 self.current_cycle = current_cycle
                 self.policy.prepare(cycle=self.current_cycle)
 
@@ -288,7 +289,7 @@ class ActorMixin(ABCActorMixin):
                 self.wrapper.tick()
 
                 self.first_tick = False
-                self.logger.debug("actor_tick: {} end".format(current_cycle))
+                self.logger.info("actor_tick: {} end".format(current_cycle))
                 current_cycle += 1
         except Exception as e:
             self.logger.debug(traceback.format_exc())
@@ -451,6 +452,9 @@ class ActorMixin(ABCActorMixin):
         """
         self.plugin.recovery_ended()
         self.policy.recovery_ended()
+        from fabric_cf.actor.core.container.globals import GlobalsSingleton
+        if GlobalsSingleton.get().can_reload_model():
+            GlobalsSingleton.get().delete_reload_model_state_file()
 
     def recover_slices(self, *, slices: List[ABCSlice]):
         """
@@ -466,6 +470,74 @@ class ActorMixin(ABCActorMixin):
                 if s.is_inventory():
                     raise e
 
+    def recover_broker_slice(self, *, slice_obj: ABCSlice):
+        """
+        Recover broker slice at the AM, do the following if the model.reload file is detected
+        - Close the existing delegations
+        - Create the new delegations from the reloaded ARM
+        - Add the delegations to the Broker Slice
+
+        @param slice_obj Slice object
+        """
+        if self.get_type() != ActorType.Authority:
+            return False
+
+        if not slice_obj.is_broker_client():
+            return False
+
+        from fabric_cf.actor.core.container.globals import GlobalsSingleton
+        if not GlobalsSingleton.get().can_reload_model():
+            return False
+
+        self.logger.info(f"Closing old delegations and adding new delegations to the slice: {slice_obj}!")
+        delegation_names = []
+
+        try:
+            delegations = self.plugin.get_database().get_delegations_by_slice_id(slice_id=slice_obj.get_slice_id())
+        except Exception as e:
+            self.logger.error(e)
+            raise ActorException(f"Could not fetch delegations records for slice {slice_obj} from database")
+
+        for d in delegations:
+            self.logger.info(f"Closing delegation: {d}!")
+            d.set_graph(graph=None)
+            d.transition(prefix="closed as part of recovers", state=DelegationState.Closed)
+            delegation_names.append(d.get_delegation_name())
+            self.plugin.get_database().update_delegation(delegation=d)
+
+        adms = self.policy.aggregate_resource_model.generate_adms()
+
+        # Create new delegations and add to the broker slice; they will be re-registered with the policy in the recovery
+        for name in delegation_names:
+            new_delegation_graph = adms.get(name)
+            dlg_obj = DelegationFactory.create(did=new_delegation_graph.get_graph_id(),
+                                               slice_id=slice_obj.get_slice_id(),
+                                               delegation_name=name)
+            dlg_obj.set_slice_object(slice_object=slice_obj)
+            dlg_obj.set_graph(graph=new_delegation_graph)
+            dlg_obj.transition(prefix="Reload Model", state=DelegationState.Delegated)
+            self.plugin.get_database().add_delegation(delegation=dlg_obj)
+
+    def recover_inventory_slice(self, *, slice_obj: ABCSlice) -> bool:
+        """
+        Check and Reload ARM for an inventory slice for an AM
+        @param slice_obj slice object
+        @return True if ARM was reloaded; otherwise False
+        """
+        if self.get_type() != ActorType.Authority:
+            return False
+
+        if not slice_obj.is_inventory():
+            return False
+
+        # Check and Reload ARM if needed
+        from fabric_cf.actor.core.container.globals import GlobalsSingleton
+        arm_graph = GlobalsSingleton.get().check_and_reload_model(graph_id=slice_obj.get_graph_id())
+        if arm_graph is not None:
+            slice_obj.set_graph(graph=arm_graph)
+
+        return arm_graph is not None
+
     def recover_slice(self, *, slice_obj: ABCSlice):
         """
         Recover slice
@@ -477,6 +549,9 @@ class ActorMixin(ABCActorMixin):
             self.logger.debug("Found slice_id: {} slice:{}".format(slice_id, slice_obj))
         else:
             self.logger.info("Recovering slice: {}".format(slice_id))
+            self.recover_inventory_slice(slice_obj=slice_obj)
+
+            self.recover_broker_slice(slice_obj=slice_obj)
 
             self.logger.debug("Informing the plugin about the slice")
             self.plugin.revisit(slice_obj=slice_obj)
@@ -563,7 +638,6 @@ class ActorMixin(ABCActorMixin):
         """
         self.logger.info(
             "Starting to recover delegations in slice {}({})".format(slice_obj.get_name(), slice_obj.get_slice_id()))
-        delegations = None
         try:
             delegations = self.plugin.get_database().get_delegations_by_slice_id(slice_id=slice_obj.get_slice_id())
         except Exception as e:
@@ -786,6 +860,9 @@ class ActorMixin(ABCActorMixin):
 
         self.message_service.start()
 
+        if self.plugin.get_handler_processor() is not None:
+            self.plugin.get_handler_processor().start()
+
     def stop(self):
         """
         Stop an actor
@@ -919,22 +996,23 @@ class ActorMixin(ABCActorMixin):
                 self.actor_main_lock.notify_all()
 
             if len(events) > 0:
-                self.logger.debug("Processing {} events".format(len(events)))
-                for e in events:
-                    self.logger.debug("Processing event of type {}".format(type(e)))
-                    self.logger.debug("Processing event {}".format(e))
+                self.logger.debug(f"Processing {len(events)} events")
+                for event in events:
+                    #self.logger.debug("Processing event of type {}".format(type(e)))
+                    #self.logger.debug("Processing event {}".format(e))
                     try:
-                        e.process()
+                        event.process()
                     except Exception as e:
-                        self.logger.error("Error while processing event {} {}".format(type(e), e))
+                        self.logger.error(f"Error while processing event {type(event)}, {e}")
                         self.logger.error(traceback.format_exc())
 
             if len(timers) > 0:
+                self.logger.debug(f"Processing {len(timers)} timers")
                 for t in timers:
                     try:
                         t.execute()
                     except Exception as e:
-                        self.logger.error("Error while processing a timer {}".format(e))
+                        self.logger.error(f"Error while processing a timer {type(t)}, {e}")
                         self.logger.error(traceback.format_exc())
 
     def setup_message_service(self):
@@ -961,9 +1039,10 @@ class ActorMixin(ABCActorMixin):
             topic = config.get_actor().get_kafka_topic()
             topics = [topic]
             consumer_conf = GlobalsSingleton.get().get_kafka_config_consumer()
-            key_schema, val_schema = GlobalsSingleton.get().get_kafka_schemas()
             self.message_service = MessageService(kafka_service=kafka_service, kafka_mgmt_service=kafka_mgmt_service,
-                                                  conf=consumer_conf, key_schema=key_schema, record_schema=val_schema,
+                                                  consumer_conf=consumer_conf,
+                                                  key_schema_location=GlobalsSingleton.get().get_config().get_kafka_key_schema_location(),
+                                                  value_schema_location=GlobalsSingleton.get().get_config().get_kafka_value_schema_location(),
                                                   topics=topics, logger=self.logger)
         except Exception as e:
             self.logger.error(traceback.format_exc())
