@@ -23,11 +23,13 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
+import threading
 from enum import Enum
 
 from fim.slivers.base_sliver import BaseSliver
+from fim.slivers.network_node import NodeSliver
 
-from fabric_cf.actor.core.apis.i_reservation import IReservation
+from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
 from fabric_cf.actor.core.plugins.handlers.config_token import ConfigToken
 from fabric_cf.actor.core.util.id import ID
 from fabric_cf.actor.core.util.notice import Notice
@@ -45,10 +47,10 @@ class UnitState(Enum):
 
 
 class Unit(ConfigToken):
-    def __init__(self, *, uid: ID, sliver: BaseSliver = None, rid: ID = None, slice_id: ID = None, actor_id: ID = None,
+    def __init__(self, *, rid: ID, sliver: BaseSliver = None, slice_id: ID = None, actor_id: ID = None,
                  properties: dict = None, state: UnitState = None, rtype: ResourceType = None):
-        # Unique identifier.
-        self.uid = uid
+        # Reservation this unit belongs to (id).
+        self.reservation_id = rid
         # Resource type.
         self.rtype = rtype
         # Unique identifier of parent unit (optional).
@@ -63,8 +65,6 @@ class Unit(ConfigToken):
             self.state = state
         # Configuration sequence number. Each unique configuration action is identified by a sequence number.
         self.sequence = 0
-        # Reservation this unit belongs to (id).
-        self.reservation_id = rid
         # Slice this unit belongs to.
         self.slice_id = slice_id
         # Actor this unit belongs to.
@@ -76,13 +76,14 @@ class Unit(ConfigToken):
         self.modified = None
         self.transfer_out_started = False
         self.sliver = sliver
-        self.worker_node_name = None
+        self.lock = threading.Lock()
 
     def __getstate__(self):
         state = self.__dict__.copy()
         del state['transfer_out_started']
         del state['modified']
         del state['reservation']
+        del state['lock']
 
         return state
 
@@ -91,6 +92,7 @@ class Unit(ConfigToken):
         self.transfer_out_started = False
         self.modified = False
         self.reservation = None
+        self.lock = threading.Lock()
 
     def transition(self, *, to_state: UnitState):
         """
@@ -112,8 +114,12 @@ class Unit(ConfigToken):
         @param message
         @param exception exception
         """
-        self.notices.add(msg=message, ex=exception)
-        self.transition(to_state=UnitState.FAILED)
+        try:
+            self.lock.acquire()
+            self.notices.add(msg=message, ex=exception)
+            self.transition(to_state=UnitState.FAILED)
+        finally:
+            self.lock.release()
 
     def fail_on_modify(self, *, message: str, exception: Exception = None):
         """
@@ -121,60 +127,88 @@ class Unit(ConfigToken):
         @param message message
         @param exception exception
         """
-        self.notices.add(msg=message, ex=exception)
-        self.transition(to_state=UnitState.ACTIVE)
-        self.merge_properties(incoming=self.modified.properties)
+        try:
+            self.lock.acquire()
+            self.notices.add(msg=message, ex=exception)
+            self.transition(to_state=UnitState.ACTIVE)
+            self.merge_properties(incoming=self.modified.properties)
+        finally:
+            self.lock.release()
 
     def set_state(self, *, state: UnitState):
         """
         Set state
         @param state state
         """
-        self.transition(to_state=state)
+        try:
+            self.lock.acquire()
+            self.transition(to_state=state)
+        finally:
+            self.lock.release()
 
     def start_close(self):
         """
         Start close on a unit
         """
-        self.transition(to_state=UnitState.CLOSING)
-        self.transfer_out_started = True
+        try:
+            self.lock.acquire()
+            self.transition(to_state=UnitState.CLOSING)
+            self.transfer_out_started = True
+        finally:
+            self.lock.release()
 
     def start_prime(self) -> bool:
         """
         Start priming the unit
         """
-        if self.state == UnitState.DEFAULT or self.state == UnitState.PRIMING:
-            self.transition(to_state=UnitState.PRIMING)
-            return True
+        try:
+            self.lock.acquire()
+            if self.state == UnitState.DEFAULT or self.state == UnitState.PRIMING:
+                self.transition(to_state=UnitState.PRIMING)
+                return True
+        finally:
+            self.lock.release()
         return False
 
     def start_modify(self) -> bool:
         """
         Start modifying
         """
-        if self.state == UnitState.ACTIVE or self.state == UnitState.MODIFYING:
-            self.transition(to_state=UnitState.MODIFYING)
-            return True
+        try:
+            self.lock.acquire()
+            if self.state == UnitState.ACTIVE or self.state == UnitState.MODIFYING:
+                self.transition(to_state=UnitState.MODIFYING)
+                return True
+        finally:
+            self.lock.release()
         return False
 
     def activate(self):
         """
         Mark the unit as active
         """
-        self.state = UnitState.ACTIVE
+        try:
+            self.lock.acquire()
+            self.state = UnitState.ACTIVE
+        finally:
+            self.lock.release()
 
     def close(self):
         """
         Mark the unit as closed
         """
-        self.state = UnitState.CLOSED
+        try:
+            self.lock.acquire()
+            self.state = UnitState.CLOSED
+        finally:
+            self.lock.release()
 
     def get_id(self) -> ID:
         """
         Get unit id
         @return unit id
         """
-        return self.uid
+        return self.reservation_id
 
     def get_properties(self) -> dict:
         """
@@ -189,7 +223,11 @@ class Unit(ConfigToken):
         @param name name
         @param value value
         """
-        self.properties[name] = value
+        try:
+            self.lock.acquire()
+            self.properties[name] = value
+        finally:
+            self.lock.release()
 
     def get_property(self, *, name: str) -> str:
         """
@@ -198,8 +236,12 @@ class Unit(ConfigToken):
         @return value
         """
         ret_val = None
-        if name in self.properties:
-            ret_val = self.properties[name]
+        try:
+            self.lock.acquire()
+            if name in self.properties:
+                ret_val = self.properties[name]
+        finally:
+            self.lock.release()
         return ret_val
 
     def get_state(self) -> UnitState:
@@ -214,7 +256,7 @@ class Unit(ConfigToken):
         Clone a unit
         @return copy of the object
         """
-        ret_val = Unit(uid=self.uid, sliver=self.sliver, properties=self.properties, state=self.state)
+        ret_val = Unit(rid=self.reservation_id, sliver=self.sliver, properties=self.properties, state=self.state)
         return ret_val
 
     def get_sequence(self) -> int:
@@ -224,20 +266,16 @@ class Unit(ConfigToken):
         """
         return self.sequence
 
-    def get_sequence_increment(self) -> int:
-        """
-        Get Sequence number and increment it
-        @return sequence number and increment it
-        """
-        self.sequence += 1
-        ret_val = self.sequence
-        return ret_val
-
     def increment_sequence(self) -> int:
         """
         Increment sequence number
         """
-        self.sequence += 1
+        try:
+            self.lock.acquire()
+            self.sequence += 1
+        finally:
+            self.lock.release()
+
         ret_val = self.sequence
         return ret_val
 
@@ -245,31 +283,47 @@ class Unit(ConfigToken):
         """
         Decrement sequence number
         """
-        self.sequence -= 1
+        try:
+            self.lock.acquire()
+            self.sequence -= 1
+        finally:
+            self.lock.release()
         ret_val = self.sequence
         return ret_val
 
-    def set_reservation(self, *, reservation: IReservation):
+    def set_reservation(self, *, reservation: ABCReservationMixin):
         """
         Set Reservation
         @param reservation reservation
         """
-        self.reservation = reservation
-        self.reservation_id = reservation.get_reservation_id()
+        try:
+            self.lock.acquire()
+            self.reservation = reservation
+            self.reservation_id = reservation.get_reservation_id()
+        finally:
+            self.lock.release()
 
     def set_slice_id(self, *, slice_id: ID):
         """
         Set slice id
         @param slice_id slice id
         """
-        self.slice_id = slice_id
+        try:
+            self.lock.acquire()
+            self.slice_id = slice_id
+        finally:
+            self.lock.release()
 
     def set_actor_id(self, *, actor_id: ID):
         """
         Set actor id
         @param actor_id actor id
         """
-        self.actor_id = actor_id
+        try:
+            self.lock.acquire()
+            self.actor_id = actor_id
+        finally:
+            self.lock.release()
 
     def is_failed(self) -> bool:
         """
@@ -307,7 +361,11 @@ class Unit(ConfigToken):
         Set modified unit
         @param modified modified
         """
-        self.modified = modified
+        try:
+            self.lock.acquire()
+            self.modified = modified
+        finally:
+            self.lock.release()
 
     def get_modified(self):
         """
@@ -330,7 +388,7 @@ class Unit(ConfigToken):
         """
         return self.reservation_id
 
-    def get_reservation(self) -> IReservation:
+    def get_reservation(self) -> ABCReservationMixin:
         """
         Get Reservation
         @return reservation
@@ -350,14 +408,22 @@ class Unit(ConfigToken):
         Set Parent id
         @param parent_id parent id
         """
-        self.parent_id = parent_id
+        try:
+            self.lock.acquire()
+            self.parent_id = parent_id
+        finally:
+            self.lock.release()
 
     def complete_modify(self):
         """
         Complete Modify operation
         """
-        self.transition(to_state=UnitState.ACTIVE)
-        self.merge_properties(incoming=self.modified.properties)
+        try:
+            self.lock.acquire()
+            self.transition(to_state=UnitState.ACTIVE)
+            self.merge_properties(incoming=self.modified.properties)
+        finally:
+            self.lock.release()
 
     def get_actor_id(self) -> ID:
         """
@@ -384,7 +450,7 @@ class Unit(ConfigToken):
         if not isinstance(other, Unit):
             return NotImplemented
 
-        return self.uid == other.uid
+        return self.reservation_id == other.reservation_id
 
     def get_notices(self) -> Notice:
         """
@@ -399,14 +465,18 @@ class Unit(ConfigToken):
         Add a notice
         @param notice notice
         """
-        self.notices.add(msg=notice)
+        try:
+            self.lock.acquire()
+            self.notices.add(msg=notice)
+        finally:
+            self.lock.release()
 
     def __str__(self):
-        return f"[unit: {self.uid} reservation: {self.reservation_id} actor: {self.actor_id} state: {self.state} " \
+        return f"[unit: {self.reservation_id} actor: {self.actor_id} state: {self.state} " \
                f"sliver: {self.sliver} properties: {self.properties}]"
 
     def __hash__(self):
-        return self.uid.__hash__()
+        return self.reservation_id.__hash__()
 
     def get_sliver(self) -> BaseSliver:
         """
@@ -421,4 +491,25 @@ class Unit(ConfigToken):
         :param sliver: sliver
         :return:
         """
-        self.sliver = sliver
+        try:
+            self.lock.acquire()
+            self.sliver = sliver
+        finally:
+            self.lock.release()
+
+    def update_sliver(self, *, sliver:BaseSliver):
+        """
+        Update the sliver associated with the Unit
+        Handler Process and AM process have different memory space
+        So it requires explicit updates to AM process object with the values updated by Handler Processes
+        Side effect of multi-processing in Python
+
+        :param sliver Sliver returned by Handler
+        """
+        try:
+            self.lock.acquire()
+            self.sliver.set_label_allocations(sliver.get_label_allocations())
+            if isinstance(self.sliver, NodeSliver) and isinstance(sliver, NodeSliver):
+                self.sliver.management_ip = sliver.management_ip
+        finally:
+            self.lock.release()
\ No newline at end of file
