@@ -34,6 +34,7 @@ from fabric_cf.actor.core.kernel.rpc_request_type import RPCRequestType
 from fabric_cf.actor.core.kernel.request_types import RequestTypes
 from fabric_cf.actor.core.kernel.reservation_server import ReservationServer
 from fabric_cf.actor.core.kernel.reservation_states import ReservationStates, ReservationPendingStates
+from fabric_cf.actor.core.util.rpc_exception import RPCError
 
 if TYPE_CHECKING:
     from fabric_cf.actor.core.apis.abc_actor_mixin import ABCActorMixin
@@ -65,6 +66,7 @@ class AuthorityReservation(ReservationServer, ABCKernelAuthorityReservationMixin
         self.notified_about_failure = False
         # Creates a new "blank" reservation instance. Used during recovery.
         self.category = ReservationCategory.Authority
+        self.broker_callback = None
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -111,6 +113,12 @@ class AuthorityReservation(ReservationServer, ABCKernelAuthorityReservationMixin
         super().restore(actor=actor, slice_obj=slice_obj)
         self.notified_about_failure = False
 
+    def set_broker_callback(self, *, broker_callback: ABCCallbackProxy):
+        self.broker_callback = broker_callback
+
+    def get_broker_callback(self) -> ABCCallbackProxy:
+        return self.broker_callback
+
     def prepare(self, *, callback: ABCCallbackProxy, logger):
         self.set_logger(logger=logger)
         self.callback = callback
@@ -259,6 +267,7 @@ class AuthorityReservation(ReservationServer, ABCKernelAuthorityReservationMixin
                                     pending=ReservationPendingStates.Priming)
                 except Exception as e:
                     self.logger.error("authority redeem e: {}".format(e))
+                    self.logger.error(traceback.format_exc())
                     self.fail_notify(message=str(e))
         elif self.state == ReservationStates.Active:
             assert extend
@@ -347,16 +356,6 @@ class AuthorityReservation(ReservationServer, ABCKernelAuthorityReservationMixin
         except Exception as e:
             self.logger.error("callback failed e:{}".format(e))
 
-    def handle_failed_rpc(self, *, failed: FailedRPC):
-        remote_auth = failed.get_remote_auth()
-        if failed.get_request_type() == RPCRequestType.UpdateLease:
-            if self.callback is None or self.callback.get_identity() != remote_auth:
-                raise AuthorityException("Unauthorized Failed reservation RPC: expected={}, but was: {}".format(
-                    self.callback.get_identity(), remote_auth))
-        else:
-            raise AuthorityException("Unexpected FailedRPC for BrokerReservation. RequestType={}".format(
-                failed.get_request_type()))
-
     def prepare_extend_lease(self):
         self.requested_resources.validate_incoming_ticket(term=self.requested_term)
 
@@ -568,6 +567,18 @@ class AuthorityReservation(ReservationServer, ABCKernelAuthorityReservationMixin
     def get_ticket(self) -> ResourceSet:
         return self.ticket
 
+    def handle_failed_rpc(self, *, failed: FailedRPC):
+        if failed.get_error_type() == RPCError.NetworkError:
+            if self.is_failed() or self.is_closed():
+                return
+
+        # Resources were allocated successfully but Orchestrator could not be informed
+        if failed.get_request_type() == RPCRequestType.UpdateLease and self.resources is not None and \
+                self.last_pending_state != ReservationPendingStates.ExtendingLease:
+            self.logger.error(f"Closing reservation due to non-recoverable RPC error {failed.get_error_type()}")
+            self.update_data.error(message=str(failed.get_error()))
+            self.actor.close(reservation=self)
+
 
 class AuthorityReservationFactory:
     """
