@@ -24,6 +24,7 @@
 #
 # Author: Komal Thareja (kthare10@renci.org)
 from fabric_cf.actor.core.apis.abc_kernel_controller_reservation_mixin import ABCKernelControllerReservationMixin
+from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
 from fabric_cf.actor.core.common.constants import Constants
 from fabric_cf.actor.core.common.exceptions import ReservationException
 
@@ -33,8 +34,15 @@ class PredecessorState:
     def __init__(self, *, reservation: ABCKernelControllerReservationMixin, filters: dict = None):
         if reservation is None:
             raise ReservationException(Constants.INVALID_ARGUMENT)
+        self.reservation_id = None
         self.reservation = reservation
-        self.filters = filters
+        if reservation is not None:
+            self.reservation_id = reservation.get_reservation_id()
+
+    def set_reservation(self, reservation: ABCReservationMixin):
+        self.reservation = reservation
+        if reservation is not None:
+            self.reservation_id = reservation.get_reservation_id()
 
     def get_reservation(self) -> ABCKernelControllerReservationMixin:
         return self.reservation
