@@ -28,17 +28,17 @@ from __future__ import annotations
 from typing import TYPE_CHECKING
 
 from datetime import datetime
-from fabric_cf.actor.core.apis.i_reservation import IReservation, ReservationCategory
-from fabric_cf.actor.core.apis.i_kernel_reservation import IKernelReservation
+from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin, ReservationCategory
+from fabric_cf.actor.core.apis.abc_kernel_reservation import ABCKernelReservation
 from fabric_cf.actor.core.common.exceptions import ReservationException
 from fabric_cf.actor.core.kernel.reservation_states import ReservationStates, ReservationPendingStates, JoinState
 from fabric_cf.actor.core.util.reservation_state import ReservationState
 
 if TYPE_CHECKING:
-    from fabric_cf.actor.core.apis.i_actor import IActor
-    from fabric_cf.actor.core.apis.i_policy import IPolicy
-    from fabric_cf.actor.core.apis.i_slice import ISlice
-    from fabric_cf.actor.core.apis.i_kernel_slice import IKernelSlice
+    from fabric_cf.actor.core.apis.abc_actor_mixin import ABCActorMixin
+    from fabric_cf.actor.core.apis.abc_policy import ABCPolicy
+    from fabric_cf.actor.core.apis.abc_slice import ABCSlice
+    from fabric_cf.actor.core.apis.abc_kernel_slice import ABCKernelSlice
     from fabric_cf.actor.core.kernel.request_types import RequestTypes
     from fabric_cf.actor.core.kernel.resource_set import ResourceSet
     from fabric_cf.actor.core.time.term import Term
@@ -46,7 +46,7 @@ if TYPE_CHECKING:
     from fabric_cf.actor.core.util.resource_type import ResourceType
 
 
-class Reservation(IKernelReservation):
+class Reservation(ABCKernelReservation):
     """
     These are the only methods synchronized on the Reservation object itself. The
     purpose is to allow an external thread to await a state transition in a
@@ -68,7 +68,7 @@ class Reservation(IKernelReservation):
     """
 
     def __init__(self, *, rid: ID = None, resources: ResourceSet = None, term: Term = None,
-                 slice_object: IKernelSlice = None):
+                 slice_object: ABCKernelSlice = None):
         # The unique reservation identifier.
         self.rid = rid
         # Reservation category. Subclasses should supply the correct value.
@@ -136,6 +136,8 @@ class Reservation(IKernelReservation):
         self.state_transition = False
         # Scratch element to trigger post-actions on a probe.
         self.service_pending = ReservationPendingStates.None_
+        self.last_transition_time = None
+        self.last_pending_state = ReservationPendingStates.None_
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -166,7 +168,7 @@ class Reservation(IKernelReservation):
         self.state_transition = False
         self.service_pending = ReservationPendingStates.None_
 
-    def restore(self, *, actor: IActor, slice_obj: ISlice):
+    def restore(self, *, actor: ABCActorMixin, slice_obj: ABCSlice):
         """
         Must be invoked after creating reservation from unpickling
         """
@@ -240,7 +242,7 @@ class Reservation(IKernelReservation):
         Modify lease on reservation
         """
 
-    def extend_ticket(self, *, actor: IActor):
+    def extend_ticket(self, *, actor: ABCActorMixin):
         """
         Extend a ticket
         """
@@ -286,7 +288,7 @@ class Reservation(IKernelReservation):
     def get_category(self) -> ReservationCategory:
         return self.category
 
-    def get_kernel_slice(self) -> IKernelSlice:
+    def get_kernel_slice(self) -> ABCKernelSlice:
         return self.slice
 
     def get_leased_abstract_units(self) -> int:
@@ -301,8 +303,8 @@ class Reservation(IKernelReservation):
 
         @return notices string
         """
-        msg = f"Reservation {self.rid} (Slice {self.slice} ) is in state [{self.get_state_name()}," \
-              f"{self.get_pending_state_name()}]"
+        msg = f"Reservation {self.rid} (Slice {self.slice}) is in state ({self.get_state_name()}," \
+              f"{self.get_pending_state_name()})"
 
         if self.error_message is not None and self.error_message != "":
             msg += f", err={self.error_message}"
@@ -345,7 +347,7 @@ class Reservation(IKernelReservation):
     def get_resources(self) -> ResourceSet:
         return self.resources
 
-    def get_slice(self) -> ISlice:
+    def get_slice(self) -> ABCSlice:
         return self.slice
 
     def get_slice_id(self):
@@ -486,7 +488,7 @@ class Reservation(IKernelReservation):
         if self.state == ReservationStates.Closed or self.state == ReservationStates.Failed:
             self.error(err="invalid Reservation")
 
-    def reserve(self, *, policy: IPolicy):
+    def reserve(self, *, policy: ABCPolicy):
         return
 
     def setup(self):
@@ -532,7 +534,7 @@ class Reservation(IKernelReservation):
     def service_update_ticket(self):
         return
 
-    def set_actor(self, *, actor: IActor):
+    def set_actor(self, *, actor: ABCActorMixin):
         self.actor = actor
 
     def set_approved(self, *, term: Term = None, approved_resources: ResourceSet = None):
@@ -571,7 +573,7 @@ class Reservation(IKernelReservation):
     def set_service_pending(self, *, code: ReservationPendingStates):
         self.service_pending = code
 
-    def set_slice(self, *, slice_object: ISlice):
+    def set_slice(self, *, slice_object: ABCSlice):
         self.slice = slice_object
 
     def __str__(self):
@@ -601,15 +603,17 @@ class Reservation(IKernelReservation):
                               f"{self.get_pending_state_name()} -> {pending.name}")
 
         self.state = state
+        self.last_pending_state = pending
         self.pending_state = pending
 
         self.set_dirty()
         self.state_transition = True
+        self.last_transition_time = datetime.utcnow()
 
-    def update_lease(self, *, incoming: IReservation, update_data):
+    def update_lease(self, *, incoming: ABCReservationMixin, update_data):
         self.internal_error(err="abstract update_lease trap")
 
-    def update_ticket(self, *, incoming: IReservation, update_data):
+    def update_ticket(self, *, incoming: ABCReservationMixin, update_data):
         self.internal_error(err="abstract update_ticket trap")
 
     def validate_outgoing(self):
@@ -663,3 +667,22 @@ class Reservation(IKernelReservation):
                 if node_map is not None:
                     return node_map[1]
         return None
+
+    def exceeds_timeout(self, timeout: int) -> bool:
+        """
+        Check if Reservation has been in the state for more than the timeout interval
+        @param timeout: timeout
+        @return True if reservation has been in the state for more than timeout; False otherwise
+        """
+        if self.last_transition_time is None:
+            return False
+
+        current_time = datetime.utcnow()
+        if (current_time - self.last_transition_time).seconds > timeout:
+            print("Return True")
+            return True
+
+        return False
+
+    def get_error_message(self) -> str:
+        return self.error_message
