@@ -24,6 +24,8 @@
 #
 # Author: Komal Thareja (kthare10@renci.org)
 from __future__ import annotations
+
+from datetime import date, datetime
 from typing import TYPE_CHECKING, List
 
 from fabric_mb.message_bus.messages.reservation_mng import ReservationMng
@@ -109,7 +111,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             self.id = actor.get_guid()
 
     def get_slices(self, *, slice_id: ID, caller: AuthToken, id_token: str = None,
-                   slice_name: str = None) -> ResultSliceAvro:
+                   slice_name: str = None, email: str = None) -> ResultSliceAvro:
         result = ResultSliceAvro()
         result.status = ResultAvro()
 
@@ -119,6 +121,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
         else:
             slice_list = None
             user_dn = None
+            user_email = email
             try:
                 if id_token is not None:
                     fabric_token = AccessChecker.check_access(action_id=ActionId.query, resource_type=ResourceType.slice,
@@ -126,6 +129,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                                                               actor_type=self.actor.get_type(),
                                                               resource_id=str(slice_id))
                     user_dn = fabric_token.get_decoded_token().get(Constants.CLAIMS_SUB, None)
+                    user_email = fabric_token.get_decoded_token().get(Constants.CLAIMS_EMAIL, None)
 
                     if user_dn is None:
                         result.status.set_code(ErrorCodes.ErrorInvalidToken.value)
@@ -140,9 +144,12 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                             slice_list = [slice_obj]
 
                     elif slice_name is not None:
-                        slice_list = self.db.get_slice_by_name(slice_name=slice_name, oidc_claim_sub=user_dn)
+                        slice_list = self.db.get_slice_by_name(slice_name=slice_name, oidc_claim_sub=user_dn,
+                                                               email=user_email)
                     elif user_dn is not None:
                         slice_list = self.db.get_slice_by_oidc_claim_sub(oidc_claim_sub=user_dn)
+                    elif user_email is not None:
+                        slice_list = self.db.get_slice_by_email(email=user_email)
                     else:
                         slice_list = self.db.get_slices()
 
@@ -174,6 +181,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
         else:
             try:
                 user_dn = None
+                user_email = None
                 if id_token is not None:
                     fabric_token = AccessChecker.check_access(action_id=ActionId.query,
                                                               resource_type=ResourceType.slice,
@@ -181,6 +189,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                                                               actor_type=self.actor.get_type(),
                                                               resource_id=str(slice_obj.slice_name))
                     user_dn = fabric_token.get_decoded_token().get(Constants.CLAIMS_SUB, None)
+                    user_email = fabric_token.get_decoded_token().get(Constants.CLAIMS_EMAIL, None)
 
                     if user_dn is None:
                         result.status.set_code(ErrorCodes.ErrorInvalidToken.value)
@@ -192,8 +201,11 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                 slice_obj_new.set_description(description=slice_obj.get_description())
                 slice_obj_new.set_owner(owner=self.actor.get_identity())
                 slice_obj_new.get_owner().set_oidc_sub_claim(oidc_sub_claim=user_dn)
+                slice_obj_new.get_owner().set_email(email=user_email)
                 slice_obj_new.set_graph_id(graph_id=slice_obj.graph_id)
                 slice_obj_new.set_config_properties(value=slice_obj.get_config_properties())
+                slice_obj_new.set_lease_end(lease_end=slice_obj.get_lease_end())
+                slice_obj_new.set_lease_start(lease_start=datetime.utcnow())
 
                 if slice_obj.get_inventory():
                     slice_obj_new.set_inventory(value=True)
@@ -306,7 +318,9 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
         return self.db.get_slice(slice_id=guid)
 
     def get_reservations(self, *, caller: AuthToken, id_token: str = None, state: int = None,
-                         slice_id: ID = None, rid: ID = None, oidc_claim_sub: str = None) -> ResultReservationAvro:
+                         slice_id: ID = None, rid: ID = None, oidc_claim_sub: str = None,
+                         email: str = None, rid_list: List[str] = None,
+                         notices_as_dict: bool = False) -> ResultReservationAvro:
         result = ResultReservationAvro()
         result.status = ResultAvro()
 
@@ -316,13 +330,15 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             return result
 
         try:
-            user_dn = None
+            user_dn = oidc_claim_sub
+            user_email = email
             if id_token is not None:
                 fabric_token = AccessChecker.check_access(action_id=ActionId.query, resource_type=ResourceType.sliver,
                                                           token=id_token, logger=self.logger,
                                                           actor_type=self.actor.get_type(),
                                                           resource_id=str(rid))
                 user_dn = fabric_token.get_decoded_token().get(Constants.CLAIMS_SUB, None)
+                user_email = fabric_token.get_decoded_token().get(Constants.CLAIMS_EMAIL, None)
 
                 if user_dn is None:
                     result.status.set_code(ErrorCodes.ErrorInvalidToken.value)
@@ -344,6 +360,15 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                     res_list = self.db.get_reservations_by_slice_id(slice_id=slice_id)
                 elif state is not None:
                     res_list = self.db.get_reservations_by_state(state=state)
+                elif user_dn is not None:
+                    res_list = self.db.get_reservations_by_oidc_claim_sub(oidc_claim_sub=user_dn)
+                elif user_email is not None:
+                    if state is None:
+                        res_list = self.db.get_reservations_by_email(email=user_email)
+                    else:
+                        res_list = self.db.get_reservations_by_email_state(email=user_email, state=state)
+                elif rid_list is not None:
+                    res_list = self.db.get_reservations_by_rids(rid=rid_list)
                 else:
                     res_list = self.db.get_reservations()
             except Exception as e:
@@ -358,7 +383,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                     slice_id = r.get_slice_id()
                     slice_obj = self.get_slice_by_guid(guid=slice_id)
                     r.restore(actor=self.actor, slice_obj=slice_obj)
-                    rr = Converter.fill_reservation(reservation=r, full=True)
+                    rr = Converter.fill_reservation(reservation=r, full=True, notices_as_dict=notices_as_dict)
                     result.reservations.append(rr)
         except ReservationNotFoundException as e:
             self.logger.error("getReservations: {}".format(e))
@@ -555,8 +580,10 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                 return result
 
         try:
-            AccessChecker.check_access(action_id=ActionId.query, resource_type=ResourceType.sliver,
-                                       token=id_token, logger=self.logger, actor_type=self.actor.get_type())
+            if id_token is not None:
+                AccessChecker.check_access(action_id=ActionId.query, resource_type=ResourceType.sliver,
+                                           token=id_token, logger=self.logger, actor_type=self.actor.get_type())
+
             res_list = None
             try:
                 res_list = self.db.get_reservations_by_rids(rid=rids)
@@ -567,27 +594,10 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                 result.status = ManagementObject.set_exception_details(result=result.status, e=e)
                 return result
 
-            if len(res_list) == len(rids):
-                result.reservation_states = []
-                for r in res_list:
-                    result.reservation_states.append(Converter.fill_reservation_state(res=r))
-            elif len(res_list) > len(rids):
-                raise ManageException("The database provided too many records")
-            else:
-                i = 0
-                j = 0
-                result.reservation_states = []
-                while i < len(rids):
-                    reservation = res_list[j]
-                    if rids[i] == reservation.get_reservation_id():
-                        result.reservation_states.append(Converter.fill_reservation_state(res=reservation))
-                        j += 1
-                    else:
-                        state = ReservationStateAvro()
-                        state.set_state(ReservationStates.Unknown.value)
-                        state.set_pending_state(ReservationPendingStates.Unknown.value)
-                        result.reservation_states.append(state)
-                    i += 1
+            result.reservation_states = []
+            for r in res_list:
+                result.reservation_states.append(Converter.fill_reservation_state(res=r))
+
         except ReservationNotFoundException as e:
             self.logger.error("get_reservation_state_for_reservations: {}".format(e))
             result.status.set_code(ErrorCodes.ErrorNoSuchReservation.value)
@@ -654,3 +664,11 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             result.status = ManagementObject.set_exception_details(result=result.status, e=e)
 
         return result
+
+    @staticmethod
+    def toggle_maintenance_mode(*, mode: bool):
+        from fabric_cf.actor.core.container.globals import GlobalsSingleton
+        if mode:
+            GlobalsSingleton.get().create_maintenance_lock()
+        else:
+            GlobalsSingleton.get().delete_maintenance_lock()
