@@ -25,6 +25,7 @@
 # Author: Komal Thareja (kthare10@renci.org)
 from __future__ import annotations
 
+import json
 import traceback
 
 from fabric_mb.message_bus.messages.add_slice_avro import AddSliceAvro
@@ -33,6 +34,7 @@ from fabric_mb.message_bus.messages.get_delegations_avro import GetDelegationsAv
 from fabric_mb.message_bus.messages.get_reservations_request_avro import GetReservationsRequestAvro
 from fabric_mb.message_bus.messages.get_reservations_state_request_avro import GetReservationsStateRequestAvro
 from fabric_mb.message_bus.messages.get_slices_request_avro import GetSlicesRequestAvro
+from fabric_mb.message_bus.messages.maintenance_request_avro import MaintenanceRequestAvro
 from fabric_mb.message_bus.messages.remove_reservation_avro import RemoveReservationAvro
 from fabric_mb.message_bus.messages.remove_slice_avro import RemoveSliceAvro
 from fabric_mb.message_bus.messages.result_avro import ResultAvro
@@ -42,7 +44,7 @@ from fabric_mb.message_bus.messages.result_reservation_state_avro import ResultR
 from fabric_mb.message_bus.messages.result_slice_avro import ResultSliceAvro
 from fabric_mb.message_bus.messages.result_string_avro import ResultStringAvro
 from fabric_mb.message_bus.messages.update_reservation_avro import UpdateReservationAvro
-from fabric_mb.message_bus.messages.message import IMessageAvro
+from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
 
 from fabric_cf.actor.core.common.constants import Constants, ErrorCodes
 from fabric_cf.actor.core.common.exceptions import ManageException
@@ -53,42 +55,45 @@ from fabric_cf.actor.core.util.id import ID
 
 
 class KafkaActorService(KafkaService):
-    def process(self, *, message: IMessageAvro):
+    def process(self, *, message: AbcMessageAvro):
         callback_topic = message.get_callback_topic()
         result = None
 
         self.logger.debug("Processing message: {}".format(message.get_message_name()))
 
-        if message.get_message_name() == IMessageAvro.get_slices_request:
+        if message.get_message_name() == AbcMessageAvro.get_slices_request:
             result = self.get_slices(request=message)
 
-        elif message.get_message_name() == IMessageAvro.remove_slice:
+        elif message.get_message_name() == AbcMessageAvro.remove_slice:
             result = self.remove_slice(request=message)
 
-        elif message.get_message_name() == IMessageAvro.add_slice:
+        elif message.get_message_name() == AbcMessageAvro.add_slice:
             result = self.add_slice(request=message)
 
-        elif message.get_message_name() == IMessageAvro.update_slice:
+        elif message.get_message_name() == AbcMessageAvro.update_slice:
             result = self.update_slice(request=message)
 
-        elif message.get_message_name() == IMessageAvro.get_reservations_request:
+        elif message.get_message_name() == AbcMessageAvro.get_reservations_request:
             result = self.get_reservations(request=message)
 
-        elif message.get_message_name() == IMessageAvro.get_delegations:
+        elif message.get_message_name() == AbcMessageAvro.get_delegations:
             result = self.get_delegations(request=message)
 
-        elif message.get_message_name() == IMessageAvro.remove_reservation:
+        elif message.get_message_name() == AbcMessageAvro.remove_reservation:
             result = self.remove_reservation(request=message)
 
-        elif message.get_message_name() == IMessageAvro.close_reservations:
+        elif message.get_message_name() == AbcMessageAvro.close_reservations:
             result = self.close_reservations(request=message)
 
-        elif message.get_message_name() == IMessageAvro.update_reservation:
+        elif message.get_message_name() == AbcMessageAvro.update_reservation:
             result = self.update_reservation(request=message)
 
-        elif message.get_message_name() == IMessageAvro.get_reservations_state_request:
+        elif message.get_message_name() == AbcMessageAvro.get_reservations_state_request:
             result = self.get_reservation_state(request=message)
 
+        elif message.get_message_name() == AbcMessageAvro.maintenance_request:
+            result = self.maintenance_request(request=message)
+
         else:
             self.logger.debug("Unsupported Message, discarding it!")
             return
@@ -96,7 +101,7 @@ class KafkaActorService(KafkaService):
         if callback_topic is None:
             self.logger.debug("No callback specified, ignoring the message")
 
-        if self.producer.produce_sync(topic=callback_topic, record=result):
+        if self.producer.produce(topic=callback_topic, record=result):
             self.logger.debug("Successfully send back response: {}".format(result.to_dict()))
         else:
             self.logger.debug("Failed to send back response: {}".format(result.to_dict()))
@@ -124,7 +129,7 @@ class KafkaActorService(KafkaService):
             if request.slice_id is not None:
                 slice_id = ID(uid=request.slice_id)
             result = mo.get_slices(slice_id=slice_id, caller=auth, id_token=request.get_id_token(),
-                                   slice_name=request.slice_name)
+                                   slice_name=request.slice_name, email=request.get_email())
 
         except Exception as e:
             self.logger.error(traceback.format_exc())
@@ -249,10 +254,11 @@ class KafkaActorService(KafkaService):
                         request.get_reservation_state() != Constants.ALL_RESERVATION_STATES:
 
                     result = mo.get_reservations(caller=auth, state=request.get_reservation_state(),
-                                                 id_token=request.get_id_token())
+                                                 id_token=request.get_id_token(), email=request.get_email())
 
                 else:
-                    result = mo.get_reservations(caller=auth, id_token=request.get_id_token())
+                    result = mo.get_reservations(caller=auth, id_token=request.get_id_token(),
+                                                 email=request.get_email())
 
         except Exception as e:
             result.status.set_code(ErrorCodes.ErrorInternalError.value)
@@ -397,3 +403,25 @@ class KafkaActorService(KafkaService):
 
         result.message_id = request.message_id
         return result
+
+    def maintenance_request(self, *, request: MaintenanceRequestAvro) -> ResultStringAvro:
+        result = ResultStringAvro()
+        result.status = ResultAvro()
+        try:
+            if request.actor_guid is None:
+                result.status.set_code(ErrorCodes.ErrorInvalidArguments.value)
+                result.status.set_message(ErrorCodes.ErrorInvalidArguments.interpret())
+                return result
+
+            mo = self.get_actor_mo(guid=ID(uid=request.actor_guid))
+            mode_str = request.get_properties().get(Constants.MODE)
+            mode = json.loads(mode_str.lower())
+            mo.toggle_maintenance_mode(mode=mode)
+
+        except Exception as e:
+            result.status.set_code(ErrorCodes.ErrorInternalError.value)
+            result.status.set_message(ErrorCodes.ErrorInternalError.interpret(exception=e))
+            result.status = ManagementObject.set_exception_details(result=result.status, e=e)
+
+        result.message_id = request.message_id
+        return result
