@@ -27,11 +27,11 @@ from __future__ import annotations
 
 from fabric_mb.message_bus.messages.get_reservation_units_request_avro import GetReservationUnitsRequestAvro
 from fabric_mb.message_bus.messages.get_unit_request_avro import GetUnitRequestAvro
-from fabric_mb.message_bus.messages.message import IMessageAvro
+from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
 from fabric_mb.message_bus.messages.result_avro import ResultAvro
 from fabric_mb.message_bus.messages.result_units_avro import ResultUnitsAvro
 
-from fabric_cf.actor.core.apis.i_reservation import ReservationCategory
+from fabric_cf.actor.core.apis.abc_reservation_mixin import ReservationCategory
 from fabric_cf.actor.core.common.constants import ErrorCodes
 from fabric_cf.actor.core.manage.kafka.services.kafka_server_actor_service import KafkaServerActorService
 from fabric_cf.actor.core.manage.management_object import ManagementObject
@@ -40,19 +40,19 @@ from fabric_cf.actor.core.util.id import ID
 
 
 class KafkaAuthorityService(KafkaServerActorService):
-    def process(self, *, message: IMessageAvro):
+    def process(self, *, message: AbcMessageAvro):
         callback_topic = message.get_callback_topic()
         result = None
 
-        if message.get_message_name() == IMessageAvro.get_reservations_request and \
+        if message.get_message_name() == AbcMessageAvro.get_reservations_request and \
                 message.get_type() is not None and \
                 message.get_type() == ReservationCategory.Authority.name:
             result = self.get_reservations_by_category(request=message, category=ReservationCategory.Authority)
 
-        elif message.get_message_name() == IMessageAvro.get_reservation_units_request:
+        elif message.get_message_name() == AbcMessageAvro.get_reservation_units_request:
             result = self.get_reservation_units(request=message)
 
-        elif message.get_message_name() == IMessageAvro.get_unit_request:
+        elif message.get_message_name() == AbcMessageAvro.get_unit_request:
             result = self.get_unit(request=message)
 
         else:
@@ -62,7 +62,7 @@ class KafkaAuthorityService(KafkaServerActorService):
         if callback_topic is None:
             self.logger.debug("No callback specified, ignoring the message")
 
-        if self.producer.produce_sync(topic=callback_topic, record=result):
+        if self.producer.produce(topic=callback_topic, record=result):
             self.logger.debug("Successfully send back response: {}".format(result.to_dict()))
         else:
             self.logger.debug("Failed to send back response: {}".format(result.to_dict()))
