@@ -30,14 +30,17 @@ import threading
 import traceback
 from datetime import datetime
 from enum import Enum
-from typing import TYPE_CHECKING, Tuple, List
+from typing import TYPE_CHECKING, Tuple, List, Any
 
 from fim.graph.abc_property_graph import ABCPropertyGraphConstants, GraphFormat
 from fim.graph.resources.abc_adm import ABCADMPropertyGraph
 from fim.pluggable import PluggableRegistry, PluggableType
+from fim.slivers.attached_components import ComponentSliver, ComponentType
 from fim.slivers.base_sliver import BaseSliver
+from fim.slivers.capacities_labels import Labels
+from fim.slivers.interface_info import InterfaceSliver, InterfaceType
 from fim.slivers.network_node import NodeSliver, NodeType
-from fim.slivers.network_link import NetworkLinkSliver
+from fim.slivers.network_service import NetworkServiceSliver, NSLayer, ServiceType
 
 from fabric_cf.actor.core.apis.abc_broker_reservation import ABCBrokerReservation
 from fabric_cf.actor.core.apis.abc_delegation import ABCDelegation
@@ -45,11 +48,15 @@ from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
 from fabric_cf.actor.core.common.constants import Constants
 from fabric_cf.actor.core.delegation.resource_ticket import ResourceTicketFactory
 from fabric_cf.actor.core.common.exceptions import BrokerException, ExceptionErrorCode
+from fabric_cf.actor.core.kernel.reservation_states import ReservationStates
 from fabric_cf.actor.core.policy.broker_calendar_policy import BrokerCalendarPolicy
 from fabric_cf.actor.core.policy.fifo_queue import FIFOQueue
+from fabric_cf.actor.core.policy.network_node_inventory import NetworkNodeInventory
+from fabric_cf.actor.core.policy.network_service_inventory import NetworkServiceInventory
 from fabric_cf.actor.core.time.actor_clock import ActorClock
 from fabric_cf.actor.core.time.term import Term
 from fabric_cf.actor.core.util.bids import Bids
+from fabric_cf.actor.core.util.id import ID
 from fabric_cf.actor.core.util.reservation_set import ReservationSet
 from fabric_cf.actor.core.policy.inventory import Inventory
 from fabric_cf.actor.core.apis.abc_client_reservation import ABCClientReservation
@@ -60,7 +67,6 @@ if TYPE_CHECKING:
     from fabric_cf.actor.core.apis.abc_broker_mixin import ABCBrokerMixin
     from fabric_cf.actor.core.policy.inventory_for_type import InventoryForType
     from fabric_cf.actor.core.util.resource_type import ResourceType
-    from fabric_cf.actor.core.kernel.resource_set import ResourceSet
 
 
 class BrokerAllocationAlgorithm(Enum):
@@ -223,9 +229,8 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         return False
 
     def formulate_bids(self, *, cycle: int) -> Bids:
-        pending = self.calendar.get_pending()
         renewing = self.calendar.get_renewing(cycle=cycle)
-        extending = self.process_renewing(renewing=renewing, pending=pending)
+        extending = self.process_renewing(renewing=renewing)
         return Bids(ticketing=ReservationSet(), extending=extending)
 
     def get_allocation(self, *, reservation: ABCBrokerReservation) -> int:
@@ -245,7 +250,8 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
 
         return start
 
-    def get_approved_term(self, *, reservation: ABCBrokerReservation) -> Term:
+    @staticmethod
+    def get_approved_term(*, reservation: ABCBrokerReservation) -> Term:
         return Term(start=reservation.get_requested_term().get_start_time(),
                     end=reservation.get_requested_term().get_end_time(),
                     new_start=reservation.get_requested_term().get_new_start_time())
@@ -273,7 +279,7 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         except Exception as e:
             self.logger.error("Exception in prepare {}".format(e))
 
-    def process_renewing(self, *, renewing: ReservationSet, pending: ReservationSet) -> ReservationSet:
+    def process_renewing(self, *, renewing: ReservationSet) -> ReservationSet:
         """
         Performs checks on renewing reservations. Updates the terms to
         suggest new terms, stores the extend on the pending list. Returns a
@@ -281,7 +287,6 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         bidding cycle.
 
         @param renewing collection of the renewing reservations
-        @param pending collection of reservations that are pending
 
         @return non-null set of renewals
         """
@@ -327,9 +332,9 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
 
         self.logger.debug(f"allocating resources for cycle {start_cycle}")
 
-        self.allocate_extending_reservation_set(requests=requests, start_cycle=start_cycle)
+        self.allocate_extending_reservation_set(requests=requests)
         self.allocate_queue(start_cycle=start_cycle)
-        self.allocate_ticketing(requests=requests, start_cycle=start_cycle)
+        self.allocate_ticketing(requests=requests)
 
     def get_default_resource_type(self) -> ResourceType:
         result = None
@@ -339,7 +344,7 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
 
         return result
 
-    def allocate_extending_reservation_set(self, *, requests: ReservationSet, start_cycle: int):
+    def allocate_extending_reservation_set(self, *, requests: ReservationSet):
         if requests is not None:
             for reservation in requests.values():
                 if reservation.is_extending_ticket() and not reservation.is_closed():
@@ -356,7 +361,7 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
                     else:
                         reservation.fail(message=Constants.NO_POOL)
 
-    def allocate_ticketing(self, *, requests: ReservationSet, start_cycle: int):
+    def allocate_ticketing(self, *, requests: ReservationSet):
         if requests is not None:
             # Holds the Node Id to List of Reservation Ids allocated
             # This is used to check on the reservations allocated during this cycle to compute available resources
@@ -366,18 +371,24 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
                 if not reservation.is_ticketing():
                     continue
 
-                status, node_id_to_reservations = self.ticket(reservation=reservation,
-                                                              node_id_to_reservations=node_id_to_reservations)
+                status, node_id_to_reservations, error_msg = self.ticket(reservation=reservation,
+                                                                         node_id_to_reservations=node_id_to_reservations)
                 if status:
                     continue
 
                 if self.queue is None and not reservation.is_failed():
-                    reservation.fail(message="Insufficient resources")
+                    fail_message = "Insufficient resources"
+                    if error_msg is not None:
+                        fail_message = error_msg
+                    reservation.fail(message=fail_message)
                     continue
 
                 if not reservation.is_failed():
-                    reservation.fail(message=f"Insufficient resources for specified start time, Failing reservation: "
-                                             f"{reservation.get_reservation_id()}")
+                    fail_message = f"Insufficient resources for specified start time, Failing reservation: " \
+                                   f"{reservation.get_reservation_id()}"
+                    if error_msg is not None:
+                        fail_message = error_msg
+                    reservation.fail(message=fail_message)
 
     def allocate_queue(self, *, start_cycle: int):
         if self.queue is None:
@@ -388,8 +399,8 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         # as the reservations are not updated in the database yet
         node_id_to_reservations = {}
         for reservation in self.queue.values():
-            status, node_id_to_reservations = self.ticket(reservation=reservation,
-                                                          node_id_to_reservations=node_id_to_reservations)
+            status, node_id_to_reservations, error_msg = self.ticket(reservation=reservation,
+                                                                     node_id_to_reservations=node_id_to_reservations)
             if not status:
                 # TODO
                 threshold = 100
@@ -401,14 +412,14 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             else:
                 self.queue.remove(reservation=reservation)
 
-    def ticket(self, *, reservation: ABCBrokerReservation, node_id_to_reservations: dict) -> Tuple[bool, dict]:
+    def ticket(self, *, reservation: ABCBrokerReservation, node_id_to_reservations: dict) -> Tuple[bool, dict, Any]:
         self.logger.debug(f"cycle: {self.actor.get_current_cycle()} new ticket request: "
                           f"{reservation}/{type(reservation).__name__}")
 
+        error_msg = None
         start = self.align_start(when=reservation.get_requested_term().get_new_start_time())
         end = self.align_end(when=reservation.get_requested_term().get_end_time())
 
-        term = None
         resource_type = reservation.get_requested_resources().get_type()
         if resource_type is None or not self.inventory.contains_type(resource_type=resource_type):
             resource_type = self.get_default_resource_type()
@@ -417,9 +428,8 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             inv = self.inventory.get(resource_type=resource_type)
 
             if inv is not None:
-                if term is None:
-                    term = Term(start=start, end=end)
-
+                self.logger.debug(f"Inventory type: {type(inv)}")
+                term = Term(start=start, end=end)
                 return self.ticket_inventory(reservation=reservation, inv=inv, term=term,
                                              node_id_to_reservations=node_id_to_reservations)
             else:
@@ -427,9 +437,9 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         else:
             reservation.fail(message=Constants.NO_POOL)
 
-        return False, node_id_to_reservations
+        return False, node_id_to_reservations, error_msg
 
-    def __candidate_nodes(self, sliver: NodeSliver) -> List[str]:
+    def __candidate_nodes(self, *, sliver: NodeSliver) -> List[str]:
         """
         Identify candidate worker nodes in this site that have at least
         as many needed components as in the sliver.
@@ -441,11 +451,186 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             props=node_props,
             comps=sliver.attached_components_info)
 
-    def __candidate_links(self, sliver: NetworkLinkSliver) -> List[str]:
-        return list()
+    def __find_first_fit(self, node_id_list: List[str], node_id_to_reservations: dict, inv: InventoryForType,
+                         reservation: ABCBrokerReservation) -> Tuple[str, BaseSliver, Any]:
+        """
+        Find First Available Node which can serve the reservation
+        @param node_id_list: Candidate Nodes
+        @param node_id_to_reservations:
+        @param inv: Inventory
+        @param reservation: Reservation
+        @return tuple containing delegation id, sliver, error message if any
+        """
+        delegation_id = None
+        sliver = None
+        error_msg = None
+        self.logger.debug(f"Possible candidates to serve {reservation} candidates# {node_id_list}")
+        requested_sliver = reservation.get_requested_resources().get_sliver()
+        for node_id in node_id_list:
+            try:
+                self.logger.debug(f"Attempting to allocate {reservation} via graph_node# {node_id}")
+                graph_node = self.get_network_node_from_graph(node_id=node_id)
+
+                if requested_sliver.labels is not None and requested_sliver.labels.instance_parent is not None:
+                    self.logger.info(f"Sliver {requested_sliver} is requested on worker: "
+                                     f"{requested_sliver.labels.instance_parent}")
+                    if graph_node.get_name() != requested_sliver.labels.instance_parent:
+                        self.logger.info(f"Skipping candidate node: {graph_node}")
+                        continue
+
+                existing_reservations = self.get_existing_reservations(node_id=node_id,
+                                                                       node_id_to_reservations=node_id_to_reservations)
+
+                delegation_id, sliver = inv.allocate(rid=reservation.get_reservation_id(),
+                                                     requested_sliver=requested_sliver,
+                                                     graph_id=self.combined_broker_model_graph_id,
+                                                     graph_node=graph_node, existing_reservations=existing_reservations)
+
+                if delegation_id is not None and sliver is not None:
+                    break
+            except BrokerException as e:
+                if e.error_code == ExceptionErrorCode.INSUFFICIENT_RESOURCES:
+                    self.logger.error(f"Exception occurred: {e}")
+                    error_msg = e.msg
+                else:
+                    raise e
+
+        if delegation_id is None and requested_sliver.labels is not None and requested_sliver.labels.instance_parent is not None:
+            error_msg = f"Insufficient Resources: {requested_sliver.labels.instance_parent} " \
+                        f"cannot serve the requested sliver"
+
+        return delegation_id, sliver, error_msg
+
+    def __allocate_nodes(self, *, reservation: ABCBrokerReservation, inv: NetworkNodeInventory, sliver: NodeSliver,
+                         node_id_to_reservations: dict) -> Tuple[str or None, BaseSliver, Any]:
+        """
+        Allocate Network Node Slivers
+        @param reservation Reservation
+        @param inv Inventory
+        @param sliver Requested sliver
+        @param node_id_to_reservations
+        @return tuple containing delegation id, sliver, error message if any
+        """
+        delegation_id = None
+        node_id_list = self.__candidate_nodes(sliver=sliver)
+
+        # no candidate nodes found
+        if len(node_id_list) == 0:
+            error_msg = f'Insufficient resources: No candidates nodes found to serve {reservation}'
+            self.logger.error(error_msg)
+            return delegation_id, sliver, error_msg
+
+        if self.get_algorithm_type().lower() == BrokerAllocationAlgorithm.FirstFit.name.lower():
+            return self.__find_first_fit(node_id_list=node_id_list,
+                                         node_id_to_reservations=node_id_to_reservations,
+                                         inv=inv, reservation=reservation)
+
+        else:
+            raise BrokerException(error_code=ExceptionErrorCode.NOT_SUPPORTED,
+                                  msg=f"Broker currently only supports First Fit")
+
+    def __allocate_services(self, *, rid: ID, inv: NetworkServiceInventory, sliver: NetworkServiceSliver,
+                            node_id_to_reservations: dict) -> Tuple[str, BaseSliver, Any]:
+        """
+        Allocate Network Service Slivers
+        @param rid Reservation Id
+        @param inv Inventory
+        @param sliver Requested sliver
+        @param node_id_to_reservations
+        @return tuple containing delegation id, sliver, error message if any
+        """
+        self.logger.debug(f"Processing Network Service sliver: {sliver}")
+        delegation_id = None
+        error_msg = None
+        owner_switch = None
+        owner_mpls_ns = None
+
+        # For each Interface Sliver;
+        for ifs in sliver.interface_info.interfaces.values():
+
+            # Fetch Network Node Id and BQM Component Id
+            node_id, bqm_component_id = ifs.get_node_map()
+            bqm_component = self.get_component_sliver(node_id=bqm_component_id)
+
+            # Get BQM Connection Point in Site Delegation (c)
+            site_cp = FimHelper.get_site_interface_sliver(component=bqm_component,
+                                                          local_name=ifs.get_labels().local_name)
+
+            self.logger.debug(f"Interface Sliver [Site Delegation] (C): {site_cp}")
+
+            # Get BQM Peer Connection Point in Site Delegation (a)
+            net_cp = self.get_net_interface_sliver(site_ifs_id=site_cp.node_id, itype=InterfaceType.TrunkPort)
+
+            if net_cp is None:
+                error_msg = "Peer Connection Point not found from Network AM"
+                raise BrokerException(msg=error_msg)
+
+            self.logger.debug(f"Peer Interface Sliver [Network Delegation] (A): {site_cp}")
+
+            # need to find the owner switch of the network service in CBM and take it's name or labels.local_name
+            owner_switch, owner_mpls_ns = self.get_owners(node_id=net_cp.node_id)
+
+            if bqm_component.get_type() == ComponentType.SharedNIC:
+                # VLAN is already set by the Orchestrator using the information from the Node Sliver Parent Reservation
+                if ifs.get_labels().vlan is None:
+                    message = "Shared NIC VLAN cannot be None"
+                    self.logger.error(message)
+                    raise BrokerException(error_code=ExceptionErrorCode.FAILURE,
+                                          msg=f"{message}")
+            else:
+                existing_reservations = self.get_existing_reservations(node_id=owner_mpls_ns.node_id,
+                                                                       node_id_to_reservations=node_id_to_reservations)
+                # Set vlan - source: (c) - only for dedicated NICs
+                ifs = inv.allocate_ifs(requested_ns=sliver, requested_ifs=ifs,
+                                       owner_switch=owner_switch, mpls_ns=owner_mpls_ns,
+                                       bqm_ifs_id=net_cp.node_id,
+                                       existing_reservations=existing_reservations)
+
+            # local_name source: (a)
+            ifs_labels = ifs.get_labels()
+            ifs_labels = Labels.update(ifs_labels, local_name=net_cp.get_name())
+
+            # NSO device name source: (a) - need to find the owner switch of the network service in CBM
+            # and take its name or labels.local_name
+            # Set the NSO device-name
+            ifs_labels = Labels.update(ifs_labels, device_name=owner_switch.get_name())
+            adm_ids = owner_switch.get_structural_info().adm_graph_ids
+            site_adm_ids = bqm_component.get_structural_info().adm_graph_ids
+
+            self.logger.debug(f"Owner MPLS Network Service: {owner_mpls_ns}")
+            self.logger.debug(f"Owner Switch: {owner_switch}")
+            self.logger.debug(f"Owner Switch: {owner_switch.network_service_info}")
+
+            net_adm_ids = [x for x in adm_ids if not x in site_adm_ids or site_adm_ids.remove(x)]
+            if len(net_adm_ids) != 1:
+                error_msg = f"More than 1 or 0 Network Delegations found! net_adm_ids: {net_adm_ids}"
+                self.logger.error(error_msg)
+                raise BrokerException(msg=error_msg)
+
+            # Update the Interface Sliver Node Map to map to (a)
+            ifs.set_node_map(node_map=(self.combined_broker_model_graph_id, net_cp.node_id))
+
+            delegation_id = net_adm_ids[0]
+
+            ifs.labels = ifs_labels
+
+            self.logger.debug(f"Allocated Interface Sliver: {ifs} delegation: {delegation_id}")
+
+        # Update the Network Service Sliver Node Map to map to parent of (a)
+        sliver.set_node_map(node_map=(self.combined_broker_model_graph_id, owner_mpls_ns.node_id))
+
+        # Set the Subnet and gateway from the Owner Switch (a)
+        if sliver.get_type() == ServiceType.FABNetv6 or sliver.get_type() == ServiceType.FABNetv4:
+            existing_reservations = self.get_existing_reservations(node_id=owner_mpls_ns.node_id,
+                                                                   node_id_to_reservations=node_id_to_reservations)
+            sliver = inv.allocate(rid=rid, requested_ns=sliver, owner_switch=owner_switch,
+                                  existing_reservations=existing_reservations)
+
+        return delegation_id, sliver, error_msg
 
     def ticket_inventory(self, *, reservation: ABCBrokerReservation, inv: InventoryForType, term: Term,
-                         node_id_to_reservations: dict) -> Tuple[bool, dict]:
+                         node_id_to_reservations: dict) -> Tuple[bool, dict, Any]:
+        error_msg = None
         try:
             rset = reservation.get_requested_resources()
             needed = rset.get_units()
@@ -458,30 +643,22 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             # intended link (and possibly interfaces connected to it)
 
             res_sliver = rset.get_sliver()
-
-            if isinstance(res_sliver, NodeSliver):
-                node_id_list = self.__candidate_nodes(res_sliver)
-            elif isinstance(res_sliver, NetworkLinkSliver):
-                node_id_list = self.__candidate_links(res_sliver)
-            else:
-                self.logger.error(f'Reservation {reservation} sliver type is neither Node, nor NetworkLink')
-                raise BrokerException(msg=f"Reservation sliver type is neither Node "
-                                      f"nor NetworkLink for reservation# {reservation}")
-
-            # no candidate nodes found
-            if len(node_id_list) == 0:
-                self.logger.error(f'No candidates nodes found to serve {reservation}')
-                return False, node_id_to_reservations
-
             delegation_id = None
             sliver = None
-            if self.get_algorithm_type().lower() == BrokerAllocationAlgorithm.FirstFit.name.lower():
-                delegation_id, sliver = self.__find_first_fit(node_id_list=node_id_list,
-                                                              node_id_to_reservations=node_id_to_reservations,
-                                                              inv=inv, reservation=reservation)
+
+            if isinstance(res_sliver, NodeSliver):
+                delegation_id, sliver, error_msg = self.__allocate_nodes(reservation=reservation, inv=inv,
+                                                                         sliver=res_sliver,
+                                                                         node_id_to_reservations=node_id_to_reservations)
+
+            elif isinstance(res_sliver, NetworkServiceSliver):
+                delegation_id, sliver, error_msg = self.__allocate_services(rid=reservation.get_reservation_id(),
+                                                                            inv=inv, sliver=res_sliver,
+                                                                            node_id_to_reservations=node_id_to_reservations)
             else:
-                raise BrokerException(error_code=ExceptionErrorCode.NOT_SUPPORTED,
-                                      msg=f"Broker currently only supports First Fit")
+                self.logger.error(f'Reservation {reservation} sliver type is neither Node, nor NetworkServiceSliver')
+                raise BrokerException(msg=f"Reservation sliver type is neither Node "
+                                      f"nor NetworkLink for reservation# {reservation}")
 
             if delegation_id is not None:
                 delegation = self.actor.get_delegation(did=delegation_id)
@@ -493,39 +670,12 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
                 if node_id_to_reservations.get(node_id, None) is None:
                     node_id_to_reservations[node_id] = ReservationSet()
                 node_id_to_reservations[node_id].add(reservation=reservation)
-                return True, node_id_to_reservations
+                return True, node_id_to_reservations, error_msg
         except Exception as e:
             self.logger.error(traceback.format_exc())
             self.logger.error(e)
             reservation.fail(message=str(e))
-        return False, node_id_to_reservations
-
-    def __find_first_fit(self, node_id_list: List[str], node_id_to_reservations: dict, inv: InventoryForType,
-                         reservation: ABCBrokerReservation) -> Tuple[str, BaseSliver]:
-        delegation_id = None
-        sliver = None
-        self.logger.debug(f"Possible candidates to serve res# {reservation} candidates# {node_id_list}")
-        for node_id in node_id_list:
-            try:
-                self.logger.debug(f"Attempting to allocate res# {reservation} via graph_node# {node_id}")
-                graph_node = self.get_node_from_graph(node_id=node_id)
-
-                existing_reservations = self.get_existing_reservations(node_id=node_id,
-                                                                       node_id_to_reservations=node_id_to_reservations)
-
-                delegation_id, sliver = inv.allocate(reservation=reservation,
-                                                     graph_id=self.combined_broker_model_graph_id,
-                                                     graph_node=graph_node, existing_reservations=existing_reservations)
-
-                if delegation_id is not None and sliver is not None:
-                    break
-            except BrokerException as e:
-                if e.error_code == ExceptionErrorCode.INSUFFICIENT_RESOURCES:
-                    self.logger.error(f"Exception occurred: {e}")
-                else:
-                    raise e
-
-        return delegation_id, sliver
+        return False, node_id_to_reservations, error_msg
 
     def extend_private(self, *, reservation: ABCBrokerReservation, inv: InventoryForType, term: Term):
         try:
@@ -541,7 +691,7 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         except Exception as e:
             self.logger.error(e)
             self.logger.error(traceback.format_exc())
-            reservation.fail(exception=e)
+            reservation.fail(message="", exception=e)
 
     def issue_ticket(self, *, reservation: ABCBrokerReservation, units: int, rtype: ResourceType,
                      term: Term, source: ABCDelegation, sliver: BaseSliver) -> ABCBrokerReservation:
@@ -573,18 +723,6 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         if isinstance(reservation, ABCBrokerReservation):
             self.logger.debug("Broker reservation")
             super().release(reservation=reservation)
-            if reservation.is_closed_in_priming():
-                self.logger.debug("Releasing resources (closed in priming)")
-                self.release_resources(rid=str(reservation.get_reservation_id()),
-                                       slice_id=str(reservation.get_slice_id()),
-                                       rset=reservation.get_approved_resources(),
-                                       term=reservation.get_approved_term())
-            else:
-                self.logger.debug("Releasing resources")
-                self.release_resources(rid=str(reservation.get_reservation_id()),
-                                       slice_id=str(reservation.get_slice_id()),
-                                       rset=reservation.get_resources(),
-                                       term=reservation.get_term())
         elif isinstance(reservation, ABCClientReservation):
             self.logger.debug("Client reservation")
             super().release(reservation=reservation)
@@ -592,17 +730,6 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             self.logger.debug(f"Removing reservation: {reservation.get_reservation_id()} "
                               f"from inventory status: {status}")
 
-    def release_resources(self, *, rid: str, slice_id: str, rset: ResourceSet, term: Term):
-        try:
-            if rset is None or term is None or rset.get_resources() is None:
-                self.logger.warning("Reservation does not have resources to release")
-                return
-            inv = self.inventory.get(resource_type=rset.get_type())
-            if inv is None:
-                raise BrokerException(msg="Cannot release resources: missing inventory")
-        except Exception as e:
-            self.logger.error(f"release resources {e}")
-
     def align_end(self, *, when: datetime) -> datetime:
         """
         Aligns the specified date with the end of the closest cycle.
@@ -697,7 +824,8 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
                 self.merge_adm(adm_graph=delegation.get_graph())
                 self.logger.debug(f"Donated Delegation: {delegation.get_delegation_id()}")
             else:
-                self.logger.warning("Delegation ignored")
+                self.logger.warning(f"Delegation ignored: {delegation.get_delegation_id()}")
+                self.logger.debug(f"Active delegations: {self.delegations}")
         except Exception as e:
             self.logger.error(f"Failed to merge ADM: {delegation}")
             self.logger.error(traceback.format_exc())
@@ -705,6 +833,23 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         finally:
             self.lock.release()
 
+    def remove_delegation(self, *, delegation: ABCDelegation):
+        try:
+            self.lock.acquire()
+            if delegation.get_delegation_id() in self.delegations:
+                self.unmerge_adm(graph_id=delegation.get_delegation_id())
+                self.delegations.pop(delegation.get_delegation_id())
+                self.logger.debug(f"Removed Delegation: {delegation.get_delegation_id()}")
+            else:
+                self.logger.warning(f"Delegation ignored: {delegation.get_delegation_id()}")
+                self.logger.debug(f"Active delegations: {self.delegations}")
+        except Exception as e:
+            self.logger.error(f"Failed to un-merge ADM: {delegation}")
+            self.logger.error(traceback.format_exc())
+            raise e
+        finally:
+            self.lock.release()
+
     def closed_delegation(self, *, delegation: ABCDelegation):
         """
         Close a delegation by un-merging from CBM
@@ -713,22 +858,70 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         :return:
         """
         self.logger.debug("Close Delegation")
+        self.remove_delegation(delegation=delegation)
+
+    def reclaim_delegation(self, *, delegation: ABCDelegation):
+        """
+        Reclaim a delegation by un-merging from CBM
+        We take snapshot of CBM before un-merge, and rollback to snapshot in case un-merge fails
+        :param delegation:
+        :return:
+        """
+        self.logger.debug("Reclaim Delegation")
+        self.remove_delegation(delegation=delegation)
+
+    def get_net_interface_sliver(self, *, site_ifs_id: str, itype: InterfaceType) -> InterfaceSliver:
+        """
+        Get Peer Interface Sliver (child of Network Service Sliver) provided node id of Interface Sliver
+        (child of Component Sliver)
+
+        E.g: Provided Connection Point which is a child of Component i.e. renc-w3-nic2-p1,
+        return Peer Connection Point i.e. HundredGigE 0/0/0/25.3
+
+        renc-w3-nic2-p1      => l10  <= HundredGigE 0/0/0/25.3
+        [Connection Point]      Link    [Connection Point]
+
+        @param site_ifs_id Interface Sliver Id
+        @param itype Interface Type
+        @return Interface sliver
+        """
         try:
             self.lock.acquire()
-            if delegation.get_delegation_id() in self.delegations:
-                self.unmerge_adm(graph_id=delegation.get_delegation_id())
-                self.delegations.pop(delegation.get_delegation_id())
-                self.logger.debug(f"Closed Delegation: {delegation.get_delegation_id()}")
-            else:
-                self.logger.warning("Delegation ignored")
-        except Exception as e:
-            self.logger.error(f"Failed to un-merge ADM: {delegation}")
-            self.logger.error(traceback.format_exc())
-            raise e
+            result = FimHelper.get_interface_sliver_by_id(ifs_node_id=site_ifs_id, graph=self.combined_broker_model,
+                                                          itype=itype)
+            if len(result) != 1:
+                raise BrokerException(msg=f"More than one Peer Interface Sliver of type {itype} found for "
+                                          f"IFS: {site_ifs_id}")
+            return next(iter(result))
         finally:
             self.lock.release()
 
-    def get_node_from_graph(self, *, node_id: str) -> NodeSliver:
+    def get_component_sliver(self, *, node_id: str) -> ComponentSliver or None:
+        """
+        Get Component Sliver from BQM
+        @param node_id: Node Id
+        @return Component Sliver
+        """
+        try:
+            self.lock.acquire()
+            if self.combined_broker_model is None:
+                return None
+            return self.combined_broker_model.build_deep_component_sliver(node_id=node_id)
+        finally:
+            self.lock.release()
+
+    def get_owners(self, *, node_id: str) -> Tuple[NodeSliver, NetworkServiceSliver]:
+        """
+        Get owner switch and network service of a Connection Point from BQM
+        @param node_id Node Id of the Connection Point
+        """
+        try:
+            self.lock.acquire()
+            return FimHelper.get_owners(bqm=self.combined_broker_model, node_id=node_id)
+        finally:
+            self.lock.release()
+
+    def get_network_node_from_graph(self, *, node_id: str) -> NodeSliver or None:
         """
         Get Node from CBM
         :param node_id:
@@ -749,8 +942,14 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
         :param node_id_to_reservations:
         :return: list of reservations
         """
-        existing_reservations = self.actor.get_plugin().get_database().get_reservations_by_graph_node_id(
-            graph_node_id=node_id)
+        states = [ReservationStates.Active.value,
+                  ReservationStates.ActiveTicketed.value,
+                  ReservationStates.Ticketed.value,
+                  ReservationStates.Nascent.value]
+
+        # Only get Active or Ticketing reservations
+        existing_reservations = self.actor.get_plugin().get_database().get_reservations_by_graph_node_id_state(
+            graph_node_id=node_id, states=states)
 
         reservations_allocated_in_cycle = node_id_to_reservations.get(node_id, None)
 
@@ -793,7 +992,8 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             self.combined_broker_model.merge_adm(adm=adm_graph)
             self.combined_broker_model.validate_graph()
             # delete the snapshot
-            self.combined_broker_model.importer.delete_graph(graph_id=snapshot_graph_id)
+            if snapshot_graph_id is not None:
+                self.combined_broker_model.importer.delete_graph(graph_id=snapshot_graph_id)
         except Exception as e:
             self.logger.error(f"Exception occurred: {e}")
             self.logger.error(traceback.format_exc())
@@ -813,7 +1013,12 @@ class BrokerSimplerUnitsPolicy(BrokerCalendarPolicy):
             if self.combined_broker_model.graph_exists():
                 snapshot_graph_id = self.combined_broker_model.snapshot()
             self.combined_broker_model.unmerge_adm(graph_id=graph_id)
-            self.combined_broker_model.validate_graph()
+            if self.combined_broker_model.graph_exists():
+                self.combined_broker_model.validate_graph()
+
+            if snapshot_graph_id is not None:
+                # delete the snapshot
+                self.combined_broker_model.importer.delete_graph(graph_id=snapshot_graph_id)
         except Exception as e:
             self.logger.error(f"Exception occurred: {e}")
             self.logger.error(traceback.format_exc())
