@@ -24,8 +24,11 @@
 #
 # Author: Komal Thareja (kthare10@renci.org)
 from enum import Enum
+
+from fabric_cf.actor.core.common.constants import Constants
 from fabric_cf.actor.core.policy.controller_simple_policy import ControllerSimplePolicy
 from fabric_cf.actor.core.util.reservation_set import ReservationSet
+from fabric_cf.actor.core.util.update_data import UpdateData
 
 
 class TicketReviewSliceState(Enum):
@@ -112,12 +115,12 @@ class ControllerTicketReviewPolicy(ControllerSimplePolicy):
 
                         # If any Reservations that are being redeemed, that means the
                         # slice has already cleared TicketReview.
-                        if slice_reservation.is_redeeming() and \
-                                slice_status_map[slice_id] == TicketReviewSliceState.Nascent:
-                            # There shouldn't be any Nascent reservations, if a reservation is being Redeemed.
-                            self.logger.error(
-                                "Nascent reservation found while Reservation {} in slice {} is redeeming"
-                                .format(slice_reservation.get_reservation_id(), slice_obj.get_name()))
+                        if slice_reservation.is_redeeming():
+                            if slice_status_map[slice_id] == TicketReviewSliceState.Nascent:
+                                # There shouldn't be any Nascent reservations, if a reservation is being Redeemed.
+                                self.logger.error(
+                                    "Nascent reservation found while Reservation {} in slice {} is redeeming"
+                                    .format(slice_reservation.get_reservation_id(), slice_obj.get_name()))
 
                             # We may have previously found a Failed Reservation,
                             # but if a ticketed reservation is being redeemed,
@@ -152,9 +155,15 @@ class ControllerTicketReviewPolicy(ControllerSimplePolicy):
                 # take action on the current reservation
                 if slice_status_map[slice_id] == TicketReviewSliceState.Failing:
                     if reservation.get_resources() is not None and reservation.get_resources().get_type() is not None:
-                        self.logger.info("Closing reservation {} due to failure in slice {}".format(
-                            reservation.get_reservation_id(), slice_obj.get_name()))
+                        msg = f"TicketReviewPolicy: Closing reservation {reservation.get_reservation_id()} due to " \
+                              f"failure in slice {slice_obj.get_name()}"
+                        self.logger.info(msg)
 
+                        if not reservation.is_failed():
+                            update_data = UpdateData()
+                            update_data.failed = True
+                            update_data.message = Constants.CLOSURE_BY_TICKET_REVIEW_POLICY
+                            reservation.mark_close_by_ticket_review(update_data=update_data)
                         self.actor.close(reservation=reservation)
                         self.calendar.remove_pending(reservation=reservation)
                         self.pending_notify.remove(reservation=reservation)
@@ -165,5 +174,13 @@ class ControllerTicketReviewPolicy(ControllerSimplePolicy):
                         .format(reservation.get_reservation_id(), slice_obj.get_name()))
                     self.pending_redeem.add(reservation=reservation)
                     self.calendar.remove_pending(reservation=reservation)
+                else:
+                    # we don't need to look at any other reservations in this slice
+                    self.logger.debug("Removing from pendingRedeem: {}".format(reservation))
+                    self.pending_redeem.remove(reservation=reservation)
+            else:
+                # Remove active or close reservations
+                self.logger.debug("Removing from pendingRedeem: {}".format(reservation))
+                self.pending_redeem.remove(reservation=reservation)
 
         super().check_pending()
