@@ -28,7 +28,7 @@ from typing import List
 from fim.slivers.attached_components import AttachedComponentsInfo
 from fim.slivers.base_sliver import BaseSliver
 from fim.slivers.network_node import NodeSliver
-from fim.user import Capacities
+from fim.user import Capacities, ComponentType
 
 from fabric_cf.actor.core.apis.abc_authority_reservation import ABCAuthorityReservation
 from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
@@ -59,9 +59,6 @@ class NetworkNodeControl(ResourceControl):
         """
         self.logger.debug(f"available_capacities: {available_capacities}")
         self.logger.debug(f"requested_capacities: {requested_capacities} for reservation# {rid}")
-        available_core = available_capacities.core
-        available_ram = available_capacities.ram
-        available_disk = available_capacities.disk
 
         # Remove allocated capacities to the reservations
         if existing_reservations is not None:
@@ -77,19 +74,17 @@ class NetworkNodeControl(ResourceControl):
                     resource_sliver = reservation.get_resources().get_sliver()
 
                 if resource_sliver is not None:
-                    self.logger.debug(f"Excluding already allocated resources {resource_sliver.get_capacities()} to "
+                    self.logger.debug(f"Excluding already allocated resources "
+                                      f"{resource_sliver.get_capacity_allocations()} to "
                                       f"reservation# {reservation.get_reservation_id()}")
-                    available_core -= resource_sliver.get_capacities().core
-                    available_ram -= resource_sliver.get_capacities().ram
-                    available_disk -= resource_sliver.get_capacities().disk
+                    # Compare the requested against available
+                    available_capacities = available_capacities - resource_sliver.get_capacity_allocations()
 
         # Compare the requested against available
-        if requested_capacities.core > available_core or requested_capacities.ram > available_ram or \
-                requested_capacities.disk > available_disk:
-            raise AuthorityException(f"Insufficient resources "
-                                  f"Cores: [{requested_capacities.core}/{available_core}] "
-                                  f"RAM: [{requested_capacities.ram}/{available_ram}] "
-                                  f"Disk: [{requested_capacities.disk}/{available_disk}]")
+        available_capacities = available_capacities - requested_capacities
+        negative_fields = available_capacities.negative_fields()
+        if len(negative_fields) > 0:
+            raise AuthorityException(f"Insufficient resources: {negative_fields}")
 
     def __check_components(self, *, rid: ID, requested_components: AttachedComponentsInfo, graph_node: BaseSliver,
                            existing_reservations: List[ABCReservationMixin]):
@@ -147,10 +142,15 @@ class NetworkNodeControl(ResourceControl):
                     for ac in allocated_components:
                         ac_node_map = ac.get_node_map()
                         if ac_node_map[1] == node_map[1]:
-                            raise AuthorityException(
-                                f"Component of type: {resource_type} BQM Node Id: {node_map[1]} "
-                                f"in graph {graph_node.node_id}"
-                                f"is already assigned to reservation# {reservation}")
+                            # For Shared NIC, make sure PCI devices are not same
+                            # For other components, having same node map is an error
+                            if (ac.get_type() == ComponentType.SharedNIC and
+                                ac.get_label_allocations().bdf == c.get_label_allocations()) or \
+                                    ac.get_type() != ComponentType.SharedNIC:
+                                raise AuthorityException(
+                                    f"Component of type: {resource_type} BQM Node Id: {node_map[1]} "
+                                    f"in graph {graph_node.node_id}"
+                                    f"is already assigned to reservation# {reservation}")
 
     def assign(self, *, reservation: ABCAuthorityReservation, delegation_name: str,
                graph_node: BaseSliver, existing_reservations: List[ABCReservationMixin]) -> ResourceSet:
@@ -188,13 +188,13 @@ class NetworkNodeControl(ResourceControl):
         if current is None:
             # Check if Capacities can be satisfied by Delegated Capacities
             self.__check_capacities(rid=reservation.get_reservation_id(),
-                                    requested_capacities=requested.get_capacities(),
+                                    requested_capacities=requested.get_capacity_allocations(),
                                     available_capacities=available_delegated_capacity,
                                     existing_reservations=existing_reservations)
 
             # Check if Capacities can be satisfied by Capacities
             self.__check_capacities(rid=reservation.get_reservation_id(),
-                                    requested_capacities=requested.get_capacities(),
+                                    requested_capacities=requested.get_capacity_allocations(),
                                     available_capacities=graph_node.get_capacities(),
                                     existing_reservations=existing_reservations)
 
@@ -226,4 +226,4 @@ class NetworkNodeControl(ResourceControl):
     def free(self, *, uset: dict):
         if uset is not None:
             for u in uset.values():
-                self.logger.debug(f"Freeing 1 unit {u}")
+                self.logger.debug(f"Freeing 1 unit {u}")
\ No newline at end of file
