@@ -31,10 +31,10 @@ from fabric_mb.message_bus.messages.extend_lease_avro import ExtendLeaseAvro
 from fabric_mb.message_bus.messages.modify_lease_avro import ModifyLeaseAvro
 from fabric_mb.message_bus.messages.redeem_avro import RedeemAvro
 from fabric_mb.message_bus.messages.reservation_avro import ReservationAvro
-from fabric_mb.message_bus.messages.message import IMessageAvro
+from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
 
 from fabric_cf.actor.core.common.exceptions import ProxyException
-from fabric_cf.actor.core.kernel.authority_reservation_factory import AuthorityReservationFactory
+from fabric_cf.actor.core.kernel.authority_reservation import AuthorityReservationFactory
 from fabric_cf.actor.core.kernel.incoming_reservation_rpc import IncomingReservationRPC
 from fabric_cf.actor.core.kernel.rpc_request_type import RPCRequestType
 from fabric_cf.actor.core.proxies.kafka.translate import Translate
@@ -42,12 +42,12 @@ from fabric_cf.actor.core.proxies.kafka.services.broker_service import BrokerSer
 from fabric_cf.actor.core.util.id import ID
 
 if TYPE_CHECKING:
-    from fabric_cf.actor.core.apis.i_authority_reservation import IAuthorityReservation
+    from fabric_cf.actor.core.apis.abc_authority_reservation import ABCAuthorityReservation
 
 
 class AuthorityService(BrokerService):
 
-    def pass_authority(self, *, reservation: ReservationAvro) -> IAuthorityReservation:
+    def pass_authority(self, *, reservation: ReservationAvro) -> ABCAuthorityReservation:
         slice_obj = Translate.translate_slice(slice_avro=reservation.slice)
         term = Translate.translate_term_from_avro(term=reservation.term)
 
@@ -115,18 +115,18 @@ class AuthorityService(BrokerService):
             raise e
         self.do_dispatch(rpc=rpc)
 
-    def process(self, *, message: IMessageAvro):
-        if message.get_message_name() == IMessageAvro.close:
+    def process(self, *, message: AbcMessageAvro):
+        if message.get_message_name() == AbcMessageAvro.close:
             self.close(request=message)
-        elif message.get_message_name() == IMessageAvro.redeem:
+        elif message.get_message_name() == AbcMessageAvro.redeem:
             self.redeem(request=message)
-        elif message.get_message_name() == IMessageAvro.extend_lease:
+        elif message.get_message_name() == AbcMessageAvro.extend_lease:
             self.extend_lease(request=message)
-        elif message.get_message_name() == IMessageAvro.modify_lease:
+        elif message.get_message_name() == AbcMessageAvro.modify_lease:
             self.modify_lease(request=message)
-        elif message.get_message_name() == IMessageAvro.result_reservation:
+        elif message.get_message_name() == AbcMessageAvro.result_reservation:
             self.logger.debug("Claim Resources Response receieved: {}".format(message))
-        elif message.get_message_name() == IMessageAvro.result_delegation:
+        elif message.get_message_name() == AbcMessageAvro.result_delegation:
             self.logger.debug("Claim Delegation Response receieved: {}".format(message))
         else:
             super().process(message=message)
\ No newline at end of file
