@@ -88,6 +88,8 @@ class Translate:
         slice_obj = SliceFactory.create(slice_id=slice_avro.guid, name=slice_avro.slice_name)
         slice_obj.set_description(description=slice_avro.description)
         slice_obj.set_config_properties(value=slice_avro.config_properties)
+        slice_obj.set_lease_start(lease_start=slice_avro.get_lease_start())
+        slice_obj.set_lease_end(lease_end=slice_avro.get_lease_end())
         return slice_obj
 
     @staticmethod
@@ -99,6 +101,8 @@ class Translate:
         avro_slice.owner = Translate.translate_auth_to_avro(auth=slice_obj.get_owner())
         avro_slice.state = slice_obj.get_state().value
         avro_slice.config_properties = slice_obj.get_config_properties()
+        avro_slice.set_lease_end(lease_end=slice_obj.get_lease_end())
+        avro_slice.set_lease_start(lease_start=slice_obj.get_lease_start())
 
         if slice_obj.get_resource_type() is not None:
             avro_slice.set_resource_type(str(slice_obj.get_resource_type()))
@@ -115,6 +119,8 @@ class Translate:
         result = AuthAvro()
         result.name = auth.get_name()
         result.guid = str(auth.get_guid())
+        result.oidc_sub_claim = auth.get_oidc_sub_claim()
+        result.email = auth.get_email()
         return result
 
     @staticmethod
@@ -125,6 +131,7 @@ class Translate:
         result.name = auth_avro.name
         result.guid = auth_avro.guid
         result.oidc_sub_claim = auth_avro.oidc_sub_claim
+        result.email = auth_avro.email
         return result
 
     @staticmethod
@@ -267,8 +274,9 @@ class Translate:
     def translate_delegation_to_avro(*, delegation: ABCDelegation) -> DelegationAvro:
         avro_delegation = DelegationAvro()
         avro_delegation.delegation_id = delegation.get_delegation_id()
+        avro_delegation.state = delegation.get_state().value
         avro_delegation.slice = Translate.translate_slice_to_avro(slice_obj=delegation.get_slice_object())
-        if delegation.get_graph() is not None:
+        if delegation.get_graph() is not None and not (delegation.is_reclaimed() or delegation.is_closed()):
             avro_delegation.graph = delegation.get_graph().serialize_graph()
         return avro_delegation
 
