@@ -23,6 +23,8 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
+from fabric_mb.message_bus.producer import AvroProducerApi
+
 from fabric_cf.actor.core.apis.abc_authority_reservation import ABCAuthorityReservation
 from fabric_cf.actor.core.apis.abc_callback_proxy import ABCCallbackProxy
 from fabric_cf.actor.core.apis.abc_controller_callback_proxy import ABCControllerCallbackProxy
@@ -39,18 +41,19 @@ from fabric_cf.actor.test.client_callback_helper import ClientCallbackHelper
 from fabric_cf.actor.test.test_exception import TestException
 
 
+class IUpdateLeaseHandler:
+    def handle_update_lease(self, *, reservation: ABCReservationMixin, update_data: UpdateData, caller: AuthToken):
+        """
+        Implemented by derived class
+        """
+
+    def check_termination(self):
+        """
+        Implemented by derived class
+        """
+
+
 class ControllerCallbackHelper(ClientCallbackHelper, ABCControllerCallbackProxy):
-    class IUpdateLeaseHandler:
-        def handle_update_lease(self, *, reservation: ABCReservationMixin, update_data: UpdateData, caller: AuthToken):
-            """
-            Implemented by derived class
-            """
-
-        def check_termination(self):
-            """
-            Implemented by derived class
-            """
-
     def __init__(self, *, name: str, guid: ID):
         super().__init__(name=name, guid=guid)
         self.called_for_lease = 0
@@ -76,7 +79,7 @@ class ControllerCallbackHelper(ClientCallbackHelper, ABCControllerCallbackProxy)
         state.update_data.absorb(other=update_data)
         return state
 
-    def execute(self, *, request: ABCRPCRequestState):
+    def execute(self, *, request: ABCRPCRequestState, producer: AvroProducerApi):
         if request.get_type() == RPCRequestType.UpdateLease:
             self.lease = request.reservation
             self.called_for_lease += 1
