@@ -23,7 +23,7 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
-from fabric_cf.actor.core.apis.i_mgmt_controller import IMgmtController
+from fabric_cf.actor.core.apis.abc_mgmt_controller_mixin import ABCMgmtControllerMixin
 from fabric_cf.actor.core.kernel.reservation_states import ReservationStates
 from fabric_cf.actor.core.util.id import ID
 from fabric_cf.orchestrator.core.exceptions import OrchestratorException
@@ -36,7 +36,7 @@ class ActiveStatusChecker(StatusChecker):
         from fabric_cf.actor.core.container.globals import GlobalsSingleton
         self.logger = GlobalsSingleton.get().get_logger()
 
-    def check_(self, *, controller: IMgmtController, rid) -> Status:
+    def check_(self, *, controller: ABCMgmtControllerMixin, rid) -> Status:
         """
         Check status
         :param controller:
