@@ -23,14 +23,17 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
+import json
 import traceback
 from datetime import datetime, timedelta
-from typing import Tuple
+from http.client import NOT_FOUND, BAD_REQUEST
+from typing import Tuple, List
 
 from fabric_mb.message_bus.messages.slice_avro import SliceAvro
-from fim.graph.resources.neo4j_cbm import Neo4jCBMGraph
+from fim.graph.resources.abc_cbm import ABCCBMPropertyGraph
+from fim.user import GraphFormat
 
-from fabric_cf.actor.core.kernel.reservation_states import ReservationStates
+from fabric_cf.actor.core.kernel.reservation_states import ReservationStates, ReservationPendingStates
 from fabric_cf.actor.core.time.actor_clock import ActorClock
 from fabric_cf.actor.fim.fim_helper import FimHelper
 from fabric_cf.actor.core.apis.abc_mgmt_controller_mixin import ABCMgmtControllerMixin
@@ -48,10 +51,10 @@ class OrchestratorHandler:
     def __init__(self):
         self.controller_state = OrchestratorKernelSingleton.get()
         from fabric_cf.actor.core.container.globals import GlobalsSingleton
-        self.logger = GlobalsSingleton.get().get_logger()
-        self.jwks_url = GlobalsSingleton.get().get_config().get_oauth_config().get(
-            Constants.PROPERTY_CONF_O_AUTH_JWKS_URL, None)
-        self.pdp_config = GlobalsSingleton.get().get_config().get_global_config().get_pdp_config()
+        self.globals = GlobalsSingleton.get()
+        self.logger = self.globals.get_logger()
+        self.jwks_url = self.globals.get_config().get_oauth_config().get(Constants.PROPERTY_CONF_O_AUTH_JWKS_URL, None)
+        self.pdp_config = self.globals.get_config().get_global_config().get_pdp_config()
 
     def get_logger(self):
         """
@@ -96,9 +99,10 @@ class OrchestratorHandler:
 
         return None
 
-    def discover_broker_query_model(self, *, controller: ABCMgmtControllerMixin, token: str,
+    def discover_broker_query_model(self, *, controller: ABCMgmtControllerMixin, token: str = None,
                                     level: int = 10, delete_graph: bool = True,
-                                    ignore_validation: bool = True) -> Tuple[str, Neo4jCBMGraph]:
+                                    ignore_validation: bool = True,
+                                    graph_format: GraphFormat = GraphFormat.GRAPHML) -> Tuple[str, ABCCBMPropertyGraph or None]:
         """
         Discover all the available resources by querying Broker
         :param controller Management Controller Object
@@ -106,27 +110,34 @@ class OrchestratorHandler:
         :param level: level of details
         :param delete_graph flag indicating if the loaded graph should be deleted or not
         :param ignore_validation flag indicating to ignore validating the graph (only needed for ADs)
-        :return tuple of dictionary containing the BQM and fim graph (if delete_graph = False)
+        :param graph_format: Graph format
+        :return tuple of dictionary containing the BQM and ABCCBMPropertyGraph (if delete_graph = False)
         """
         broker = self.get_broker(controller=controller)
         if broker is None:
             raise OrchestratorException("Unable to determine broker proxy for this controller. "
                                         "Please check Orchestrator container configuration and logs.")
 
-        model = controller.get_broker_query_model(broker=broker, id_token=token, level=level)
+        model = controller.get_broker_query_model(broker=broker, id_token=token, level=level,
+                                                  graph_format=graph_format)
         if model is None:
             raise OrchestratorException(f"Could not discover types: {controller.get_last_error()}")
 
-        graph_ml = model.get_model()
+        graph_str = model.get_model()
+
         try:
-            if graph_ml is not None and graph_ml != '':
-                graph = FimHelper.get_neo4j_cbm_graph_from_string_direct(graph_str=graph_ml,
-                                                                         ignore_validation=ignore_validation)
-                if delete_graph:
-                    FimHelper.delete_graph(graph_id=graph.get_graph_id())
-                return graph_ml, graph
+            if graph_str is not None and graph_str != '':
+                graph = None
+                # Load graph only when GraphML
+                if graph_format == GraphFormat.GRAPHML:
+                    graph = FimHelper.get_neo4j_cbm_graph_from_string_direct(graph_str=graph_str,
+                                                                             ignore_validation=ignore_validation)
+                    if delete_graph:
+                        FimHelper.delete_graph(graph_id=graph.get_graph_id())
+                        graph = None
+                return graph_str, graph
             else:
-                raise OrchestratorException(http_error_code=OrchestratorException.HTTP_NOT_FOUND,
+                raise OrchestratorException(http_error_code=NOT_FOUND,
                                             message="Resource(s) not found!")
         except Exception as e:
             self.logger.error(traceback.format_exc())
@@ -154,6 +165,39 @@ class OrchestratorHandler:
             self.logger.error(f"Exception occurred processing list_resources e: {e}")
             raise e
 
+    def portal_list_resources(self, *, graph_format_str: str) -> dict:
+        """
+        List Resources
+        :param graph_format_str: Graph format
+        :raises Raises an exception in case of failure
+        :returns Broker Query Model on success
+        """
+        try:
+            controller = self.controller_state.get_management_actor()
+            self.logger.debug(f"portal_list_resources invoked controller:{controller}")
+
+            broker_query_model = None
+            graph_format = self.__translate_graph_format(graph_format=graph_format_str)
+
+            saved_bqm = self.controller_state.get_saved_bqm(graph_format=graph_format)
+            if saved_bqm is not None and not saved_bqm.can_refresh():
+                broker_query_model = saved_bqm.get_bqm()
+
+            if broker_query_model is None:
+                broker_query_model, graph = self.discover_broker_query_model(controller=controller,
+                                                                             ignore_validation=True,
+                                                                             level=1,
+                                                                             graph_format=graph_format)
+                if broker_query_model is not None:
+                    self.controller_state.save_bqm(bqm=broker_query_model, graph_format=graph_format)
+
+            return ResponseBuilder.get_broker_query_model_summary(bqm=broker_query_model)
+
+        except Exception as e:
+            self.logger.error(traceback.format_exc())
+            self.logger.error(f"Exception occurred processing portal_list_resources e: {e}")
+            raise e
+
     def create_slice(self, *, token: str, slice_name: str, slice_graph: str, ssh_key: str,
                      lease_end_time: str) -> dict:
         """
@@ -166,15 +210,15 @@ class OrchestratorHandler:
         :raises Raises an exception in case of failure
         :returns List of reservations created for the Slice on success
         """
+        if self.globals.is_maintenance_mode_on():
+            raise OrchestratorException(Constants.MAINTENANCE_MODE_ERROR)
+
         slice_id = None
         controller = None
         orchestrator_slice = None
-        bqm_graph = None
         asm_graph = None
         try:
-            end_time = None
-            if lease_end_time is not None:
-                end_time = self.__validate_lease_end_time(lease_end_time=lease_end_time)
+            end_time = self.__validate_lease_end_time(lease_end_time=lease_end_time)
 
             controller = self.controller_state.get_management_actor()
             self.logger.debug(f"create_slice invoked for Controller: {controller}")
@@ -189,9 +233,6 @@ class OrchestratorHandler:
 
             asm_graph = FimHelper.get_neo4j_asm_graph(slice_graph=slice_graph)
 
-            bqm_string, bqm_graph = self.discover_broker_query_model(controller=controller, token=token,
-                                                                     delete_graph=False)
-
             broker = self.get_broker(controller=controller)
             if broker is None:
                 raise OrchestratorException("Unable to determine broker proxy for this controller. "
@@ -202,7 +243,8 @@ class OrchestratorHandler:
             slice_obj.set_client_slice(True)
             slice_obj.set_description("Description")
             slice_obj.graph_id = asm_graph.get_graph_id()
-            slice_obj.config_properties = {Constants.USER_SSH_KEY: ssh_key}
+            slice_obj.set_config_properties(value={Constants.USER_SSH_KEY: ssh_key})
+            slice_obj.set_lease_end(lease_end=end_time)
 
             self.logger.debug(f"Adding Slice {slice_name}")
             slice_id = controller.add_slice(slice_obj=slice_obj, id_token=token)
@@ -220,14 +262,14 @@ class OrchestratorHandler:
 
             # Create Slivers from Slice Graph; Compute Reservations from Slivers;
             # Add Reservations to relational database;
-            computed_reservations = orchestrator_slice.create(bqm_graph=bqm_graph, slice_graph=asm_graph,
-                                                              end_time=end_time)
+            computed_reservations = orchestrator_slice.create(slice_graph=asm_graph)
 
             # Process the Slice i.e. Demand the computed reservations i.e. Add them to the policy
             # Once added to the policy; Actor Tick Handler will do following asynchronously:
             # 1. Ticket message exchange with broker and
             # 2. Redeem message exchange with AM once ticket is granted by Broker
-            self.controller_state.get_sdt().process_slice(controller_slice=orchestrator_slice)
+            self.controller_state.demand_slice(controller_slice=orchestrator_slice)
+            # self.controller_state.get_sdt().process_slice(controller_slice=orchestrator_slice)
 
             return ResponseBuilder.get_reservation_summary(res_list=computed_reservations)
         except Exception as e:
@@ -238,12 +280,10 @@ class OrchestratorHandler:
             self.logger.error(f"Exception occurred processing create_slice e: {e}")
             raise e
         finally:
-            if bqm_graph is not None:
-                FimHelper.delete_graph(graph_id=bqm_graph.get_graph_id())
             if orchestrator_slice is not None:
                 orchestrator_slice.unlock()
 
-    def get_slivers(self, *, token: str, slice_id: str, sliver_id: str = None, include_notices: bool = False) -> dict:
+    def get_slivers(self, *, token: str, slice_id: str, sliver_id: str = None, include_notices: bool = True) -> dict:
         """
         Get Slivers for a Slice
         :param token Fabric Identity Token
@@ -270,13 +310,13 @@ class OrchestratorHandler:
                     self.logger.error(controller.get_last_error())
                     if controller.get_last_error().status.code == ErrorCodes.ErrorNoSuchSlice:
                         raise OrchestratorException(f"Slice# {slice_id} not found",
-                                                    http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+                                                    http_error_code=NOT_FOUND)
                     elif controller.get_last_error().status.code == ErrorCodes.ErrorNoSuchReservation:
                         raise OrchestratorException(f"Reservation# {rid} not found",
-                                                    http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+                                                    http_error_code=NOT_FOUND)
 
                 raise OrchestratorException(f"Slice# {slice_id} has no reservations",
-                                            http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+                                            http_error_code=NOT_FOUND)
 
             return ResponseBuilder.get_reservation_summary(res_list=reservations, include_notices=include_notices,
                                                            include_sliver=True)
@@ -285,12 +325,13 @@ class OrchestratorHandler:
             self.logger.error(f"Exception occurred processing get_slivers e: {e}")
             raise e
 
-    def get_slices(self, *, token: str, slice_id: str = None) -> dict:
+    def get_slices(self, *, token: str, slice_id: str = None, states: List[str] = None) -> dict:
         """
         Get User Slices
         :param token Fabric Identity Token
         :param slice_id Slice Id
-=       :raises Raises an exception in case of failure
+        :param states Slice states
+        :raises Raises an exception in case of failure
         :returns List of Slices on success
         """
         try:
@@ -301,14 +342,37 @@ class OrchestratorHandler:
             if slice_id is not None:
                 slice_guid = ID(uid=slice_id)
 
-            slice_list = controller.get_slices(id_token=token, slice_id=slice_guid)
-            if slice_list is None or len(slice_list) == 0:
-                if controller.get_last_error() is not None:
-                    self.logger.error(controller.get_last_error())
-                raise OrchestratorException(f"User# has no Slices",
-                                            http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+            slice_states = SliceState.str_list_to_state_list(states=states)
 
-            return ResponseBuilder.get_slice_summary(slice_list=slice_list, slice_id=slice_id)
+            slice_list = controller.get_slices(id_token=token, slice_id=slice_guid)
+
+            error_message = {}
+            reservations_states_to_peek = [ReservationStates.Failed, ReservationStates.Closed,
+                                           ReservationStates.CloseWait]
+            for s in slice_list:
+                slice_state = SliceState(s.get_state())
+                if slice_state != SliceState.StableOK:
+                    reservations = controller.get_reservations(id_token=token, slice_id=ID(uid=s.get_slice_id()))
+                    if reservations is not None:
+                        reservations_status = {}
+                        for r in reservations:
+                            res_state = ReservationStates(r.get_state())
+                            res_pending_state = ReservationPendingStates(r.get_pending_state())
+                            if res_state in reservations_states_to_peek:
+                                status = {
+                                    ResponseBuilder.PROP_RESERVATION_STATE: str(res_state),
+                                    ResponseBuilder.PROP_RESERVATION_PENDING_STATE: str(res_pending_state)
+                                }
+                                if r.get_notices() is not None and len(r.get_notices()) > 0:
+                                    res_notices = json.loads(r.get_notices())
+                                    for k, v in res_notices.items():
+                                        status[k] = v
+                                reservations_status[r.get_reservation_id()] = status
+                        if len(reservations_status) > 0:
+                            error_message[s.get_slice_id()] = reservations_status
+
+            return ResponseBuilder.get_slice_summary(slice_list=slice_list, slice_id=slice_id,
+                                                     slice_states=slice_states, error_message=error_message)
         except Exception as e:
             self.logger.error(traceback.format_exc())
             self.logger.error(f"Exception occurred processing get_slices e: {e}")
@@ -333,14 +397,14 @@ class OrchestratorHandler:
 
             if slice_list is None or len(slice_list) == 0:
                 raise OrchestratorException(f"Slice# {slice_id} not found",
-                                            http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+                                            http_error_code=NOT_FOUND)
 
             slice_object = next(iter(slice_list))
 
             slice_state = SliceState(slice_object.get_state())
             if slice_state == SliceState.Dead or slice_state == SliceState.Closing:
                 raise OrchestratorException(f"Slice# {slice_id} already closed",
-                                            http_error_code=OrchestratorException.HTTP_BAD_REQUEST)
+                                            http_error_code=BAD_REQUEST)
 
             if slice_state != SliceState.StableOK and slice_state != SliceState.StableError:
                 self.logger.info(f"Unable to delete Slice# {slice_guid} that is not yet stable, try again later")
@@ -354,12 +418,13 @@ class OrchestratorHandler:
             self.logger.error(f"Exception occurred processing delete_slice e: {e}")
             raise e
 
-    def get_slice_graph(self, *, token: str, slice_id: str) -> dict:
+    def get_slice_graph(self, *, token: str, slice_id: str, graph_format_str: str) -> dict:
         """
         Get User Slice
         :param token Fabric Identity Token
         :param slice_id Slice Id
-=       :raises Raises an exception in case of failure
+        :param graph_format_str
+        :raises Raises an exception in case of failure
         :returns Slice Graph on success
         """
         try:
@@ -375,7 +440,7 @@ class OrchestratorHandler:
                 if controller.get_last_error() is not None:
                     self.logger.error(controller.get_last_error())
                 raise OrchestratorException(f"User# has no Slices",
-                                            http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+                                            http_error_code=NOT_FOUND)
 
             slice_obj = next(iter(slice_list))
 
@@ -384,10 +449,17 @@ class OrchestratorHandler:
 
             slice_model = FimHelper.get_graph(graph_id=slice_obj.get_graph_id())
 
+            graph_format = self.__translate_graph_format(graph_format=graph_format_str)
+            if graph_format == GraphFormat.JSON_NODELINK:
+                slice_model_str = slice_model.serialize_graph()
+                slice_model = FimHelper.get_networkx_graph_from_string(graph_str=slice_model_str)
+
             if slice_model is None:
                 raise OrchestratorException(f"Slice# {slice_obj} graph could not be loaded")
 
-            return ResponseBuilder.get_slice_model_summary(slice_model=slice_model.serialize_graph())
+            slice_model_str = slice_model.serialize_graph(format=graph_format)
+            return ResponseBuilder.get_slice_summary(slice_list=slice_list, slice_id=slice_id,
+                                                     slice_model=slice_model_str)
         except Exception as e:
             self.logger.error(traceback.format_exc())
             self.logger.error(f"Exception occurred processing get_slice_graph e: {e}")
@@ -402,6 +474,9 @@ class OrchestratorHandler:
         :raises Raises an exception in case of failure
         :return:
         """
+        if self.globals.is_maintenance_mode_on():
+            raise OrchestratorException(Constants.MAINTENANCE_MODE_ERROR)
+
         failed_to_extend_rid_list = []
         try:
             controller = self.controller_state.get_management_actor()
@@ -415,14 +490,14 @@ class OrchestratorHandler:
 
             if slice_list is None or len(slice_list) == 0:
                 raise OrchestratorException(f"Slice# {slice_id} not found",
-                                            http_error_code=OrchestratorException.HTTP_NOT_FOUND)
+                                            http_error_code=NOT_FOUND)
 
             slice_object = next(iter(slice_list))
 
             slice_state = SliceState(slice_object.get_state())
             if slice_state == SliceState.Dead or slice_state == SliceState.Closing:
                 raise OrchestratorException(f"Slice# {slice_id} already closed",
-                                            http_error_code=OrchestratorException.HTTP_BAD_REQUEST)
+                                            http_error_code=BAD_REQUEST)
 
             if slice_state != SliceState.StableOK and slice_state != SliceState.StableError:
                 self.logger.info(f"Unable to renew Slice# {slice_guid} that is not yet stable, try again later")
@@ -455,6 +530,10 @@ class OrchestratorHandler:
                                                        new_end_time=new_end_time)
                 if not result:
                     failed_to_extend_rid_list.append(r.get_reservation_id())
+                else:
+                    slice_object.set_lease_end(lease_end=new_end_time)
+                    if not controller.update_slice(slice_obj=slice_object):
+                        self.logger.error(f"Failed to update lease end time: {new_end_time} in Slice: {slice_object}")
 
             return ResponseBuilder.get_response_summary(rid_list=failed_to_extend_rid_list)
         except Exception as e:
@@ -470,16 +549,19 @@ class OrchestratorHandler:
         :raises Exception if new end time is in past
         """
         new_end_time = None
+        if lease_end_time is None:
+            new_end_time = datetime.utcnow() + timedelta(hours=Constants.DEFAULT_LEASE_IN_HOURS)
+            return new_end_time
         try:
-            new_end_time = datetime.strptime(lease_end_time, Constants.RENEW_TIME_FORMAT)
+            new_end_time = datetime.strptime(lease_end_time, Constants.LEASE_TIME_FORMAT)
         except Exception as e:
-            raise OrchestratorException(f"Lease End Time is not in format {Constants.RENEW_TIME_FORMAT}",
-                                        http_error_code=OrchestratorException.HTTP_BAD_REQUEST)
+            raise OrchestratorException(f"Lease End Time is not in format {Constants.LEASE_TIME_FORMAT}",
+                                        http_error_code=BAD_REQUEST)
 
         now = datetime.utcnow()
         if new_end_time <= now:
             raise OrchestratorException(f"New term end time {new_end_time} is in the past! ",
-                                        http_error_code=OrchestratorException.HTTP_BAD_REQUEST)
+                                        http_error_code=BAD_REQUEST)
 
         if (new_end_time - now) > Constants.DEFAULT_MAX_DURATION:
             self.logger.info(f"New term end time {new_end_time} exceeds system default "
@@ -488,3 +570,14 @@ class OrchestratorHandler:
             new_end_time = now + Constants.DEFAULT_MAX_DURATION
 
         return new_end_time
+
+    @staticmethod
+    def __translate_graph_format(*, graph_format: str) -> GraphFormat:
+        if graph_format == GraphFormat.GRAPHML.name:
+            return GraphFormat.GRAPHML
+        elif graph_format == GraphFormat.JSON_NODELINK.name:
+            return GraphFormat.JSON_NODELINK
+        elif graph_format == GraphFormat.CYTOSCAPE.name:
+            return GraphFormat.CYTOSCAPE
+        else:
+            return GraphFormat.GRAPHML
