@@ -29,7 +29,9 @@ from fabric_mb.message_bus.messages.lease_reservation_avro import LeaseReservati
 from fabric_mb.message_bus.messages.reservation_mng import ReservationMng
 from fabric_mb.message_bus.messages.slice_avro import SliceAvro
 from fim.slivers.base_sliver import BaseSliver
+from fim.slivers.json import JSONSliver
 from fim.slivers.network_node import NodeSliver
+from fim.slivers.network_service import NetworkServiceSliver
 
 from fabric_cf.actor.core.common.constants import Constants
 from fabric_cf.actor.core.kernel.reservation_states import ReservationStates, ReservationPendingStates, JoinState
@@ -52,8 +54,8 @@ class ResponseBuilder:
     PROP_SLICE_NAME = "slice_name"
     PROP_SLICE_STATE = "slice_state"
     PROP_RESERVATION_ID = "reservation_id"
-    PROP_RESOURCE_TYPE = "resource_type"
     PROP_RESERVATION_STATE = "reservation_state"
+    PROP_LEASE_START_TIME = "lease_start"
     PROP_LEASE_END_TIME = "lease_end"
     PROP_RESERVATION_PENDING_STATE = "pending_state"
     PROP_RESERVATION_JOIN_STATE = "join_state"
@@ -61,13 +63,11 @@ class ResponseBuilder:
     PROP_GRAPH_NODE_ID = "graph_node_id"
     PROP_GRAPH_ID = "graph_id"
     PROP_NAME = "name"
-    PROP_MANAGEMENT_IP = "management_ip"
-    PROP_SITE = "site"
-    PROP_CAPACITIES = "capacities"
-    PROP_LABELS = "labels"
+    PROP_SLIVER = "sliver"
+    PROP_SLIVER_TYPE = "sliver_type"
 
     @staticmethod
-    def get_reservation_summary(*, res_list: List[ReservationMng], include_notices: bool = False,
+    def get_reservation_summary(*, res_list: List[ReservationMng], include_notices: bool = True,
                                 include_sliver: bool = False) -> dict:
         """
         Get Reservation summary
@@ -84,11 +84,10 @@ class ResponseBuilder:
             for reservation in res_list:
                 res_dict = {ResponseBuilder.PROP_SLICE_ID: reservation.get_slice_id(),
                             ResponseBuilder.PROP_RESERVATION_ID: reservation.get_reservation_id(),
-                            ResponseBuilder.PROP_RESOURCE_TYPE: reservation.get_resource_type(),
                             ResponseBuilder.PROP_RESERVATION_STATE: ReservationStates(reservation.get_state()).name}
 
                 if reservation.get_pending_state() is not None:
-                    res_dict[ResponseBuilder.PROP_RESERVATION_JOIN_STATE] = \
+                    res_dict[ResponseBuilder.PROP_RESERVATION_PENDING_STATE] = \
                         ReservationPendingStates(reservation.get_pending_state()).name
 
                 if isinstance(reservation, LeaseReservationAvro) and reservation.get_join_state() is not None:
@@ -98,14 +97,19 @@ class ResponseBuilder:
                 if sliver is not None:
                     res_dict[ResponseBuilder.PROP_GRAPH_NODE_ID] = sliver.node_id
                     if include_sliver:
-                        res_dict = ResponseBuilder.get_sliver_json(sliver=sliver, result=res_dict)
+                        res_dict[ResponseBuilder.PROP_SLIVER_TYPE] = type(sliver).__name__
+                        res_dict[ResponseBuilder.PROP_SLIVER] = JSONSliver.sliver_to_json(sliver=sliver)
 
                 if include_notices:
                     res_dict[ResponseBuilder.PROP_NOTICES] = reservation.get_notices()
 
+                if reservation.get_start() is not None:
+                    start_time = ActorClock.from_milliseconds(milli_seconds=reservation.get_start())
+                    res_dict[ResponseBuilder.PROP_LEASE_START_TIME] = start_time.strftime(Constants.LEASE_TIME_FORMAT)
+
                 if reservation.get_end() is not None:
                     end_time = ActorClock.from_milliseconds(milli_seconds=reservation.get_end())
-                    res_dict[ResponseBuilder.PROP_LEASE_END_TIME] = end_time.strftime(Constants.RENEW_TIME_FORMAT)
+                    res_dict[ResponseBuilder.PROP_LEASE_END_TIME] = end_time.strftime(Constants.LEASE_TIME_FORMAT)
 
                 reservations.append(res_dict)
         else:
@@ -119,11 +123,16 @@ class ResponseBuilder:
         return response
 
     @staticmethod
-    def get_slice_summary(*, slice_list: List[SliceAvro], slice_id: str = None) -> dict:
+    def get_slice_summary(*, slice_list: List[SliceAvro], slice_id: str = None,
+                          slice_states: List[SliceState] = None, slice_model: str = None,
+                          error_message: dict = None) -> dict:
         """
         Get slice summary
         :param slice_list:
         :param slice_id:
+        :param slice_states:
+        :param slice_model:
+        :param error_message:
         :return:
         """
         slices = []
@@ -133,15 +142,24 @@ class ResponseBuilder:
         if slice_list is not None:
             for s in slice_list:
                 slice_state = SliceState(s.get_state())
-                if slice_id is None and (slice_state == SliceState.Dead or slice_state == SliceState.Closing):
+                if slice_id is None and slice_states is not None and slice_state not in slice_states:
                     continue
                 s_dict = {ResponseBuilder.PROP_SLICE_ID: s.get_slice_id(),
                           ResponseBuilder.PROP_SLICE_NAME: s.get_slice_name(),
                           ResponseBuilder.PROP_GRAPH_ID: s.get_graph_id(),
-                          ResponseBuilder.PROP_SLICE_STATE: slice_state.name}
+                          ResponseBuilder.PROP_SLICE_STATE: slice_state.name,
+                          }
+                end_time = s.get_lease_end()
+                if end_time is not None:
+                    s_dict[ResponseBuilder.PROP_LEASE_END_TIME] = end_time.strftime(Constants.LEASE_TIME_FORMAT)
+
+                if slice_model is not None:
+                    s_dict[ResponseBuilder.RESPONSE_SLICE_MODEL] = slice_model
+
+                if error_message is not None and s.get_slice_id() in error_message:
+                    s_dict[ResponseBuilder.PROP_NOTICES] = error_message.get(s.get_slice_id())
                 slices.append(s_dict)
         else:
-            status = ResponseBuilder.STATUS_FAILURE
             message = "No slices were found"
 
         response = {ResponseBuilder.RESPONSE_STATUS: status,
@@ -170,37 +188,31 @@ class ResponseBuilder:
 
         return response
 
-    @staticmethod
-    def get_slice_model_summary(*, slice_model: str):
-        """
-        Get Slice Model summary
-        :param slice_model:
-        :return:
-        """
-        status = ResponseBuilder.STATUS_OK
-        message = ""
-
-        if slice_model is None:
-            message = "No slice model found"
-            status = ResponseBuilder.STATUS_FAILURE
-
-        response = {ResponseBuilder.RESPONSE_STATUS: status,
-                    ResponseBuilder.RESPONSE_MESSAGE: message,
-                    ResponseBuilder.RESPONSE_SLICE_MODEL: slice_model}
-
-        return response
-
     @staticmethod
     def get_sliver_json(*, sliver: BaseSliver, result: dict) -> dict:
+        result[ResponseBuilder.PROP_NAME] = sliver.get_name()
+        result[ResponseBuilder.PROP_SITE] = sliver.get_site()
+        result[ResponseBuilder.PROP_RESOURCE_TYPE] = str(sliver.get_type())
+        if sliver.get_capacities() is not None:
+            result[ResponseBuilder.PROP_CAPACITIES] = sliver.get_capacities().to_json()
+        if sliver.get_capacity_allocations() is not None:
+            result[ResponseBuilder.PROP_ALLOCATED_CAPACITIES] = sliver.get_capacity_allocations().to_json()
+        if sliver.get_labels() is not None:
+            result[ResponseBuilder.PROP_LABELS] = sliver.get_labels().to_json()
+        if sliver.get_label_allocations() is not None:
+            result[ResponseBuilder.PROP_ALLOCATED_LABELS] = sliver.get_label_allocations().to_json()
+
+        # Network Node Specific Fields
         if isinstance(sliver, NodeSliver):
-            result[ResponseBuilder.PROP_NAME] = sliver.get_name()
-            result[ResponseBuilder.PROP_SITE] = sliver.get_site()
             if sliver.get_management_ip() is not None:
                 result[ResponseBuilder.PROP_MANAGEMENT_IP] = sliver.get_management_ip()
-            if sliver.get_capacities() is not None:
-                result[ResponseBuilder.PROP_CAPACITIES] = sliver.get_capacities().to_json()
-            if sliver.get_labels() is not None:
-                result[ResponseBuilder.PROP_LABELS] = sliver.get_labels().to_json()
+            if sliver.get_capacity_hints() is not None:
+                result[ResponseBuilder.PROP_CAPACITY_HINTS] = sliver.get_capacity_hints().to_json()
+
+        # Network Service Specific Fields
+        if isinstance(sliver, NetworkServiceSliver):
+            print("TODO: populate NetworkServiceSliver fields")
+
         return result
 
     @staticmethod
