@@ -23,14 +23,19 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
+from http.client import INTERNAL_SERVER_ERROR, OK
+
 import connexion
 import requests
 import six
+from fss_utils.http_errors import cors_response
 
 from fabric_cf.orchestrator.swagger_server.models.version import Version  # noqa: E501
-from fabric_cf.orchestrator.swagger_server import util, received_counter, success_counter, failure_counter
+from fabric_cf.orchestrator.swagger_server import received_counter, success_counter, failure_counter
 from fabric_cf.orchestrator.swagger_server.response.constants import VERSIONS_PATH, GET_METHOD
 
+from fabric_cf import __VERSION__
+
 
 def version_get():  # noqa: E501
     """version
@@ -44,16 +49,16 @@ def version_get():  # noqa: E501
     from fabric_cf.actor.core.container.globals import GlobalsSingleton
     logger = GlobalsSingleton.get().get_logger()
     try:
-        version = '1.0.0'
-        tag = '1.0.0'
-        url = "https://api.github.com/repos/fabric-testbesd/ControlFramework/git/refs/tags/{}".format(tag)
+        version = __VERSION__
+        tag = f"rel{__VERSION__}"
+        url = "https://api.github.com/repos/fabric-testbed/ControlFramework/git/ref/tags/{}".format(tag)
 
         response = Version()
         response.version = version
         response.gitsha1 = 'Not Available'
 
         result = requests.get(url)
-        if result.status_code == 200 and result.json() is not None:
+        if result.status_code == OK and result.json() is not None:
             object_json = result.json().get("object", None)
             if object_json is not None:
                 sha = object_json.get("sha", None)
@@ -63,5 +68,5 @@ def version_get():  # noqa: E501
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, VERSIONS_PATH).inc()
-        return str(e), 500
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=str(e))
     return response
