@@ -2,15 +2,16 @@ from datetime import datetime
 
 import connexion
 import six
+from http.client import INTERNAL_SERVER_ERROR
+from fss_utils.http_errors import cors_response
 
-from fabric_cf.actor.core.common.constants import Constants
 from fabric_cf.orchestrator.core.exceptions import OrchestratorException
 from fabric_cf.orchestrator.core.orchestrator_handler import OrchestratorHandler
 from fabric_cf.orchestrator.swagger_server import received_counter, success_counter, failure_counter
 from fabric_cf.orchestrator.swagger_server.models.success import Success  # noqa: E501
 from fabric_cf.orchestrator.swagger_server.response.constants import POST_METHOD, SLICES_CREATE_PATH, \
     SLICES_GET_SLICE_ID_PATH, GET_METHOD, SLICES_GET_PATH, DELETE_METHOD, SLICES_DELETE_PATH, SLICES_RENEW_PATH, \
-    INTERNAL_SERVER_ERROR, BAD_REQUEST, SLICE_STATUS_SLICE_ID_PATH
+    SLICE_STATUS_SLICE_ID_PATH
 from fabric_cf.orchestrator.swagger_server.response.utils import get_token
 
 
@@ -52,11 +53,13 @@ def slices_create_post(body, slice_name, ssh_key, lease_end_time):  # noqa: E501
     except OrchestratorException as e:
         logger.exception(e)
         failure_counter.labels(POST_METHOD, SLICES_CREATE_PATH).inc()
-        return str(e), e.get_http_error_code()
+        msg = str(e).replace("\n", "")
+        return cors_response(status=e.get_http_error_code(), xerror=str(e), body=msg)
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(POST_METHOD, SLICES_CREATE_PATH).inc()
-        return str(e), INTERNAL_SERVER_ERROR
+        msg = str(e).replace("\n", "")
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=msg)
 
 
 def slices_delete_slice_iddelete(slice_id):  # noqa: E501
@@ -82,18 +85,22 @@ def slices_delete_slice_iddelete(slice_id):  # noqa: E501
     except OrchestratorException as e:
         logger.exception(e)
         failure_counter.labels(DELETE_METHOD, SLICES_DELETE_PATH).inc()
-        return str(e), e.get_http_error_code()
+        msg = str(e).replace("\n", "")
+        return cors_response(status=e.get_http_error_code(), xerror=str(e), body=msg)
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(DELETE_METHOD, SLICES_DELETE_PATH).inc()
-        return str(e), INTERNAL_SERVER_ERROR
+        msg = str(e).replace("\n", "")
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=msg)
 
 
-def slices_get():  # noqa: E501
+def slices_get(states):  # noqa: E501
     """Retrieve a listing of user slices
 
     Retrieve a listing of user slices # noqa: E501
 
+    :param states: Slice states
+    :type states: List[str]
 
     :rtype: Success
     """
@@ -102,7 +109,7 @@ def slices_get():  # noqa: E501
     received_counter.labels(GET_METHOD, SLICES_GET_PATH).inc()
     try:
         token = get_token()
-        value = handler.get_slices(token=token)
+        value = handler.get_slices(token=token, states=states)
         response = Success()
         response.value = value
         success_counter.labels(GET_METHOD, SLICES_GET_PATH).inc()
@@ -110,11 +117,13 @@ def slices_get():  # noqa: E501
     except OrchestratorException as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, SLICES_GET_PATH).inc()
-        return str(e), e.get_http_error_code()
+        msg = str(e).replace("\n", "")
+        return cors_response(status=e.get_http_error_code(), xerror=str(e), body=msg)
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, SLICES_GET_PATH).inc()
-        return str(e), INTERNAL_SERVER_ERROR
+        msg = str(e).replace("\n", "")
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=msg)
 
 
 def slices_modify_slice_idput(body, slice_id):  # noqa: E501
@@ -181,14 +190,16 @@ def slices_renew_slice_idpost(slice_id, new_lease_end_time):  # noqa: E501
     except OrchestratorException as e:
         logger.exception(e)
         failure_counter.labels(POST_METHOD, SLICES_RENEW_PATH).inc()
-        return str(e), e.get_http_error_code()
+        msg = str(e).replace("\n", "")
+        return cors_response(status=e.get_http_error_code(), xerror=str(e), body=msg)
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(POST_METHOD, SLICES_RENEW_PATH).inc()
-        return str(e), INTERNAL_SERVER_ERROR
+        msg = str(e).replace("\n", "")
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=msg)
 
 
-def slices_slice_idget(slice_id):  # noqa: E501
+def slices_slice_idget(slice_id, graph_format):  # noqa: E501
     """slice properties
 
     Retrieve Slice properties # noqa: E501
@@ -203,7 +214,7 @@ def slices_slice_idget(slice_id):  # noqa: E501
     received_counter.labels(GET_METHOD, SLICES_GET_SLICE_ID_PATH).inc()
     try:
         token = get_token()
-        value = handler.get_slice_graph(token=token, slice_id=slice_id)
+        value = handler.get_slice_graph(token=token, slice_id=slice_id, graph_format_str=graph_format)
         response = Success()
         response.value = value
         success_counter.labels(GET_METHOD, SLICES_GET_SLICE_ID_PATH).inc()
@@ -211,11 +222,13 @@ def slices_slice_idget(slice_id):  # noqa: E501
     except OrchestratorException as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, SLICES_GET_SLICE_ID_PATH).inc()
-        return str(e), e.get_http_error_code()
+        msg = str(e).replace("\n", "")
+        return cors_response(status=e.get_http_error_code(), xerror=str(e), body=msg)
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, SLICES_GET_SLICE_ID_PATH).inc()
-        return str(e), INTERNAL_SERVER_ERROR
+        msg = str(e).replace("\n", "")
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=msg)
 
 
 def slices_status_slice_idget(slice_id):  # noqa: E501
@@ -244,8 +257,10 @@ def slices_status_slice_idget(slice_id):  # noqa: E501
     except OrchestratorException as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, SLICE_STATUS_SLICE_ID_PATH).inc()
-        return str(e), e.get_http_error_code()
+        msg = str(e).replace("\n", "")
+        return cors_response(status=e.get_http_error_code(), xerror=str(e), body=msg)
     except Exception as e:
         logger.exception(e)
         failure_counter.labels(GET_METHOD, SLICE_STATUS_SLICE_ID_PATH).inc()
-        return str(e), INTERNAL_SERVER_ERROR
+        msg = str(e).replace("\n", "")
+        return cors_response(status=INTERNAL_SERVER_ERROR, xerror=str(e), body=msg)
