@@ -12,6 +12,19 @@ from fabric_cf.orchestrator.swagger_server.test import BaseTestCase
 class TestResourcesController(BaseTestCase):
     """ResourcesController integration test stubs"""
 
+    def test_portalresources_get(self):
+        """Test case for portalresources_get
+
+        Retrieve a listing and description of available resources for portal
+        """
+        query_string = [('graph_format', 'GRAPHML')]
+        response = self.client.open(
+            '//portalresources',
+            method='GET',
+            query_string=query_string)
+        self.assert200(response,
+                       'Response body is : ' + response.data.decode('utf-8'))
+
     def test_resources_get(self):
         """Test case for resources_get
 
