@@ -36,7 +36,7 @@ class TestSlicesController(BaseTestCase):
         Delete slice.
         """
         response = self.client.open(
-            '//slices/delete/{sliceID}'.format(sliceID='slice_id_example'),
+            '//slices/delete/{sliceID}'.format(slice_id='slice_id_example'),
             method='DELETE')
         self.assert200(response,
                        'Response body is : ' + response.data.decode('utf-8'))
@@ -46,9 +46,11 @@ class TestSlicesController(BaseTestCase):
 
         Retrieve a listing of user slices
         """
+        query_string = [('states', 'states_example')]
         response = self.client.open(
             '//slices',
-            method='GET')
+            method='GET',
+            query_string=query_string)
         self.assert200(response,
                        'Response body is : ' + response.data.decode('utf-8'))
 
@@ -59,7 +61,7 @@ class TestSlicesController(BaseTestCase):
         """
         body = 'body_example'
         response = self.client.open(
-            '//slices/modify/{sliceID}'.format(sliceID='slice_id_example'),
+            '//slices/modify/{sliceID}'.format(slice_id='slice_id_example'),
             method='PUT',
             data=json.dumps(body),
             content_type='text/plain')
@@ -72,7 +74,7 @@ class TestSlicesController(BaseTestCase):
         Redeem resources reserved via Create API
         """
         response = self.client.open(
-            '//slices/redeem/{sliceID}'.format(sliceID='slice_id_example'),
+            '//slices/redeem/{sliceID}'.format(slice_id='slice_id_example'),
             method='POST')
         self.assert200(response,
                        'Response body is : ' + response.data.decode('utf-8'))
@@ -84,7 +86,7 @@ class TestSlicesController(BaseTestCase):
         """
         query_string = [('new_lease_end_time', 'new_lease_end_time_example')]
         response = self.client.open(
-            '//slices/renew/{sliceID}'.format(sliceID='slice_id_example'),
+            '//slices/renew/{sliceID}'.format(slice_id='slice_id_example'),
             method='POST',
             query_string=query_string)
         self.assert200(response,
@@ -95,9 +97,11 @@ class TestSlicesController(BaseTestCase):
 
         slice properties
         """
+        query_string = [('graph_format', 'GRAPHML')]
         response = self.client.open(
-            '//slices/{sliceID}'.format(sliceID='slice_id_example'),
-            method='GET')
+            '//slices/{sliceID}'.format(slice_id='slice_id_example'),
+            method='GET',
+            query_string=query_string)
         self.assert200(response,
                        'Response body is : ' + response.data.decode('utf-8'))
 
@@ -107,7 +111,7 @@ class TestSlicesController(BaseTestCase):
         slice status
         """
         response = self.client.open(
-            '//slices/status/{sliceID}'.format(sliceID='slice_id_example'),
+            '//slices/status/{sliceID}'.format(slice_id='slice_id_example'),
             method='GET')
         self.assert200(response,
                        'Response body is : ' + response.data.decode('utf-8'))
